#!/usr/bin/env python
from __future__ import division, with_statement

"""
validate.py:

Provides command-line and package entry points for analyzing the segment
length distribution in a provided BED-formatted segmentation.

"""

# A package-unique, descriptive module name used for filenames, etc
# Must be the same as the folder containing this script
MODULE="length_distribution"

__version__ = "$Revision: 89 $"

import os
import sys

from collections import defaultdict
from numpy import array, concatenate, median
from rpy2.robjects import r, numpy2ri

# XXX: Do this without the kludgy constants
from .common import die, get_ordered_labels, image_saver, LABEL_ALL, load_segmentation, make_htmlfilename, make_namebase_summary, make_pngfilename, make_tabfilename, make_thumbfilename, map_mnemonics, r_source, SEGMENT_START_COL, SEGMENT_END_COL, SEGMENT_LABEL_KEY_COL, setup_directory, tab_saver

from .html import save_html_div

FIELDNAMES_SUMMARY = ["label", "num.segs", "mean.len", "median.len",
                     "stdev.len", "num.bp", "frac.bp"]
FIELDNAMES = ["label", "length"]
NAMEBASE = "%s" % MODULE
NAMEBASE_SUMMARY = make_namebase_summary(NAMEBASE)
NAMEBASE_SIZES = "segment_sizes"
TEMPLATE_FILENAME = "length_div.tmpl"

HTML_TITLE = "Length distribution"

SIZE_PNG_WIDTH = 800
SIZE_PNG_HEIGHT = 600
PNG_WIDTH = 600
PNG_HEIGHT_BASE = 100  # Axes and label
PNG_HEIGHT_PER_LABEL = 45

def start_R():
    r_source("common.R")
    r_source("length.R")

## Given a segmentation, returns the length of each segment
def segmentation_lengths(segmentation):
    # key: label_key
    # val: list of numpy.ndarray
    lengths = defaultdict(list)
    labels = segmentation.labels

    # convert segment coords to lengths
    for segments in segmentation.chromosomes.itervalues():
        for label_key in labels.iterkeys():
            # XXX: Do this without the kludgy constants
            labeled_row_indexes = (segments[:, SEGMENT_LABEL_KEY_COL] == label_key)
            labeled_rows = segments[labeled_row_indexes]
            lengths[label_key].append(labeled_rows[:, SEGMENT_END_COL] -
                                      labeled_rows[:, SEGMENT_START_COL])

    # key: label_key
    # val: int
    num_bp = {}

    # convert lengths to:
    # key: label_key
    # val: numpy.ndarray(dtype=integer)
    for label_key, label_lengths_list in lengths.iteritems():
        label_lengths = concatenate(label_lengths_list)
        lengths[label_key] = label_lengths
        num_bp[label_key] = label_lengths.sum()

    return lengths, num_bp

## Specifies the format of a row of the summary tab file
def make_summary_row(label, lengths, num_bp, total_bp):
    return {"label": label,
            "num.segs": len(lengths),
            "mean.len": "%.3f" % lengths.mean(),
            "median.len": "%.3f" % median(lengths),
            "stdev.len": "%.3f" % lengths.std(),
            "num.bp": num_bp,
            "frac.bp": "%.3f" % (num_bp / total_bp)}

## Saves the length summary data to a tab file, using mnemonics if specified
def save_summary_tab(lengths, labels, num_bp, dirpath,
                     clobber=False, mnemonics=None):
    # Get mnemonic ordering and labels, if specified
    ordered_keys, labels = get_ordered_labels(labels, mnemonics)
    with tab_saver(dirpath, NAMEBASE_SUMMARY, FIELDNAMES_SUMMARY,
                   clobber=clobber) as saver:
        # "all" row first
        total_bp = sum(num_bp.itervalues())
        all_lengths = concatenate(lengths.values())
        row = make_summary_row(LABEL_ALL, all_lengths,
                               total_bp, total_bp)
        saver.writerow(row)

        # Remaining label rows
        for label_key in ordered_keys:
            label = labels[label_key]
            label_lengths = lengths[label_key]
            num_bp_label = num_bp[label_key]
            row = make_summary_row(label, label_lengths,
                                   num_bp_label, total_bp)
            saver.writerow(row)

def make_row(label, length):
    return {"label": label,
            "length": length}
        
def save_tab(lengths, labels, num_bp, dirpath, clobber=False):
    # fix order for iterating through dict; must be consistent
    label_keys = sorted(labels.keys())

    # XXXopt: allocating space in advance might be faster
    lengths_array = concatenate([lengths[label_key]
                                 for label_key in label_keys])

    # creates an array that has the label repeated as many times as
    # the length
    labels_array = concatenate([[labels[label_key]] * len(lengths[label_key])
                               for label_key in label_keys])
    
    with tab_saver(dirpath, NAMEBASE, FIELDNAMES, clobber=clobber) as saver:
        for label, length in zip(labels_array, lengths_array):
            row = make_row(label, length)
            saver.writerow(row)

## Generates and saves an R plot of the length distributions
def save_plot(num_labels, dirpath, clobber=False, mnemonics=[]):
    start_R()

    # Load data from corresponding tab file
    tabfilename = make_tabfilename(dirpath, NAMEBASE)
    if not os.path.isfile(tabfilename):
        die("Unable to find tab file: %s" % tabfilename)

    with image_saver(dirpath, NAMEBASE,
                     width=PNG_WIDTH,
                     height=num_labels * PNG_HEIGHT_PER_LABEL + PNG_HEIGHT_BASE,
                     clobber=clobber):
        r.plot(r["plot.length"](tabfilename, mnemonics=mnemonics))

## Generates and saves an R plot of the length distributions
def save_size_plot(dirpath, clobber=False):
    start_R()

    # Load data from corresponding tab file
    tabfilename = make_tabfilename(dirpath, NAMEBASE_SUMMARY)
    if not os.path.isfile(tabfilename):
        die("Unable to find tab file: %s" % tabfilename)

    with image_saver(dirpath, NAMEBASE_SIZES,
                     width=SIZE_PNG_WIDTH,
                     height=SIZE_PNG_HEIGHT,
                     clobber=clobber):
        r.plot(r["plot.segment.sizes"](tabfilename))

def save_html(dirpath, clobber=False):
    extra_namebases = {"sizes": NAMEBASE_SIZES}
    save_html_div(TEMPLATE_FILENAME, dirpath, namebase=NAMEBASE,
                  extra_namebases=extra_namebases,
                  tablenamebase=NAMEBASE_SUMMARY, module=MODULE,
                  clobber=clobber, title=HTML_TITLE)

## Package entry point
def validate(bedfilename, dirpath, clobber=False, replot=False, noplot=False,
             mnemonicfilename=None):
    setup_directory(dirpath)
    segmentation = load_segmentation(bedfilename)
    
    assert segmentation is not None
    
    labels = segmentation.labels
    mnemonics = map_mnemonics(labels, mnemonicfilename)

    if not replot:
        lengths, num_bp=segmentation_lengths(segmentation)
        save_tab(lengths, labels, num_bp, dirpath, clobber=clobber)
        save_summary_tab(lengths, labels, num_bp, dirpath,
                         clobber=clobber, mnemonics=mnemonics)

    if not noplot:
        save_plot(len(labels), dirpath, clobber=clobber,
                  mnemonics=mnemonics)
        save_size_plot(dirpath, clobber=clobber)

    save_html(dirpath, clobber=clobber)
    
def parse_options(args):
    from optparse import OptionParser

    usage = "%prog [OPTIONS] BEDFILE"
    version = "%%prog %s" % __version__
    parser = OptionParser(usage=usage, version=version)
    
    parser.add_option("--clobber", action="store_true",
                      dest="clobber", default=False,
                      help="Overwrite existing output files if the specified"
                      " directory already exists.")
    parser.add_option("--replot", action="store_true", 
                      dest="replot", default=False,
                      help="Load data from output tab files and"
                      " regenerate plots instead of recomputing data")
    parser.add_option("--noplot", action="store_true", 
                      dest="noplot", default=False,
                      help="Do not generate plots")

    parser.add_option("--mnemonic-file", dest="mnemonicfilename",
                      default=None,
                      help="If specified, labels will be shown using"
                      " mnemonics found in this file")
    parser.add_option("-o", "--outdir", 
                      dest="outdir", default="%s" % MODULE,
                      help="File output directory (will be created"
                      " if it does not exist) [default: %default]")

    (options, args) = parser.parse_args(args)

    if len(args) < 1:
        parser.error("Insufficient number of arguments")

    if options.noplot and options.replot:
        parser.error("noplot and replot are contradictory")
        
    return (options, args)
    
## Command-line entry point
def main(args=sys.argv[1:]):
    (options, args) = parse_options(args)
    bedfilename = args[0]

    validate(bedfilename, options.outdir, clobber=options.clobber,
             replot=options.replot, noplot=options.noplot,
             mnemonicfilename=options.mnemonicfilename)
        
if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
from __future__ import division, with_statement

"""
validate.py:

Provides command-line and package entry points for analyzing the signal
distribution over tracks and labels.

"""

# A package-unique, descriptive module name used for filenames, etc
# Must be the same as the folder containing this script
MODULE="signal_distribution"

__version__ = "$Revision: 90 $"


import os
import sys

from collections import defaultdict
#from contextlib import contextmanager
from itertools import repeat
from functools import partial
from numpy import array, concatenate, histogram, isfinite, nanmax, nanmin, \
    NINF, PINF, where
from rpy2.robjects import r, numpy2ri

# XXX: Do this without the kludgy constants
from .common import die, fill_array, \
    get_supercontig_splice, \
    image_saver, LABEL_ALL, load_segmentation, load_genome, \
    loop_segments_continuous, make_tabfilename, \
    map_mnemonics, map_segments, r_source, \
    SEGMENT_START_COL, SEGMENT_END_COL, SEGMENT_LABEL_KEY_COL, \
    setup_directory, tab_saver

from .html import save_html_div

FIELDNAMES = ["label", "trackname", "lower_edge", "count"]
NAMEBASE = "%s" % MODULE

HTML_TITLE = "Signal value distribution"
HTML_TEMPLATE_FILENAME = "signal_div.tmpl"

PNG_WIDTH_PER_SUBPLOT = 200
PNG_HEIGHT_PER_SUBPLOT = 150
PNG_BASE_HEIGHT = 200
PNG_BASE_WIDTH = 100

NUM_BINS = 100


def constant_factory(val):
    return repeat(val).next

def remove_nans(numbers):
# by Mirela
    return numbers[isfinite(numbers)]

## Returns a dict of trackname -> tuples: (min, max), one for each trackname
def seg_min_max(chromosome, segmentation, verbose=False):
    #print >>sys.stderr, "Looking for min & max in chromosome ", chromosome.name, " for track ", track_index
    #print >>sys.stderr, "segmentation: ", segmentation.chromosomes
    
    tracknames = chromosome.tracknames_continuous
    limits = dict([(trackname, (PINF, NINF)) for trackname in tracknames])
    
    if chromosome.name not in segmentation.chromosomes.keys():
        return limits
    
    for segment, continuous in \
            loop_segments_continuous(chromosome, segmentation,
                                     verbose=verbose):
        if continuous.shape[0] > 0:
            for trackname in tracknames:
                col_index = chromosome.index_continuous(trackname)
                continuous_col = continuous[:, col_index]
                
                if len(continuous_col) != 0:
                    old_min, old_max = limits[trackname]
                    cur_min = nanmin(continuous_col, axis=0)
                    cur_max = nanmax(continuous_col, axis=0)
                    limits[trackname] = (min(old_min, cur_min),
                                         max(old_max, cur_max))
                    
    return limits        

def get_num_tracks(genome):    
    # MA: this should really be part of the Genome class - TODO
    for chromosome in genome:    
        tracknames = chromosome.tracknames_continuous
        return len(tracknames)

## Loads the ranges of each track from the genomedata object
##   unless a segmentation is specified, in which case the ranges
##   are calculated for that segmentation
## Thus, defaults to loading ranges for all tracks from genomedata
def load_track_ranges(genome, segmentation=None):
    """
    returns a dict
    key: trackname
    val: (min, max) for that trackname
    """
    # start with the most extreme possible values
    res = defaultdict(constant_factory((PINF, NINF)))
    for chromosome in genome:
        if segmentation is not None:
            print >>sys.stderr, "\t%s" % chromosome.name
            res_chrom = seg_min_max(chromosome, segmentation)
            for trackname, limits in res_chrom.iteritems():
                old_min, old_max = res[trackname]
                cur_min, cur_max = limits
                res[trackname] = (min(old_min, cur_min), max(old_max, cur_max))
                #print >>sys.stderr, "Limits for %s: %s" % (trackname, res[trackname]) 
        else: 
            tracknames = chromosome.tracknames_continuous
            for trackname in tracknames:
                col_index = chromosome.index_continuous(trackname)
                cur_min = chromosome.mins[col_index]
                cur_max = chromosome.maxs[col_index]
                        
                old_min, old_max = res[trackname]
                res[trackname] = (min(old_min, cur_min), max(old_max, cur_max))
	
    # Cast out of defaultdict
    return dict(res)

# def print_array(tag, arr, type="%d"):
#     fstring = "%%s:  %s, %s, ..., %s, ..., %s, %s" % tuple([type]*5)
#     print fstring % (tag,
#                      arr[0],
#                      arr[1],
#                      arr[int(arr.shape[0] / 2)],
#                      arr[-2],
#                      arr[-1])

## Computes a set of histograms, one for each track-label pair.
def calc_histogram(genome, segmentation, num_bins=None,
                   group_labels=False, calc_ranges=False,
                   value_range=(None, None), quick=False):
    """
    if num_bins is:
    - None: There will be a bin for every possible value
    - > 0: There will be this many bins
      
    if group_labels is:
    - True: Group together all labels and only generate one histogram
      for the track over all labels
    - False: Generate a histogram for every track-label pair
      
    if calc_ranges is:
    - False: use the precomputed ranges stored for the entire genome file
    - True: calculate the segmentation ranges across the entire input

    value_range is a (min, max) tuple:
      if either value is None, it is ignored
      if min/max is not None, it is used as a limit for all binning
      if both are specified, calc_ranges is ignored
    
    if quick is:
    - True: the histogram is calculated for only the first chromosome
    
    returns a dict

    if group_labels:
      key: trackname
      val: [hist, edges]
    else:
      key: label_key
      val: dict
           key: trackname
           val: [hist, edges]
    """
    assert genome is not None
    assert segmentation is not None

    if calc_ranges and any(lim is None for lim in value_range):
        # Calculate ranges over segmentation only
        print >>sys.stderr, "Calculating track ranges...",
        track_ranges = load_track_ranges(genome, segmentation)
        print >>sys.stderr, "done."
    else:
        # Lookup ranges in genomedata
        track_ranges = load_track_ranges(genome)
        
    # Override ranges as necessary
    value_min, value_max = value_range
    for trackname, track_range in track_ranges.iteritems():
        track_min, track_max = track_range
        if value_min is not None:
            track_min = value_min
        if value_max is not None:
            track_max = value_max
        track_ranges[trackname] = (track_min, track_max)

    tracks = track_ranges.keys()  # Re-select just the tracks used

    if len(tracks) == 0:
        die("Trying to calculate histogram for no tracks")

        
    # Calculate bins for each track
    if num_bins is None:  # Have a bin for every possible integer value
        track_bins = dict([(trackname,
                            math.ceil(track_range[1]) -
                            math.floor(track_range[0]))
                           for trackname, track_range in track_ranges])
    else:  # Use the given number of bins
        assert num_bins > 0
        track_bins = dict([(trackname, num_bins) for trackname in tracks])

        
    # key: trackname
    # val: histogram_func: a function that generates a histogram from data
    #                      with uniform bins within a track
    # For every trackname, there's a function that generates histograms
    histogram_funcs = dict((trackname, partial(histogram,
                                               bins=track_bins[trackname],
                                               range=track_ranges[trackname],
                                               new=True))
                           for trackname in tracks)
    # Trackname is one the track names, track_range is (min,max)
    # functools.partial(func[, *args][, **keywords])
    #   Returns a new partial object which when called will behave like func
    #   called with args and keywords.
    
    labels = segmentation.labels
    if group_labels:  # Pretend there is just one "all" label
        labels = dict([(-1, "all")])
        
    # Return dictionary: see function docstring for specification
    # Now for every label and for every trackname
    #   there's a histogram array of the right boundary and frequency
    res = dict((label_key, dict((trackname,
                                 list(histogram_func(array([]))))
                                for trackname, histogram_func
                                in histogram_funcs.iteritems()))
               for label_key in labels)
    
    print >>sys.stderr, "Generating signal distribution histograms"
    num_tracks = len(tracks)
    num_labels = len(labels)
    for chromosome in genome:
        chrom = chromosome.name
        print >>sys.stderr, "\t%s" % chrom

        # Iterate through supercontigs and segments together
        for segment, continuous_seg in \
                loop_segments_continuous(chromosome, segmentation):

            if group_labels:
                cur_res = res  # All labels are used
            else:
                seg_label_key = segment[SEGMENT_LABEL_KEY_COL]
                cur_res = res[seg_label_key]
                
            # Iterate through each track
            for trackname, histogram_func in histogram_funcs.iteritems():
                # col_index is the index of the trackname
                #   (if 5 tracks, it can be 0, 1, 2, 3 or 4)
                col_index = chromosome.index_continuous(trackname)
                
                # continuous_col is the "intensity"
                #   (continuous number) from the data track
                # len(supercontig_map) = len(continuous_col)
                #   because they are for the same segment
                continuous_col = continuous_seg[:, col_index]
                
                # Remove the NaN's, otherwise the numpy.histogram function 
                #   gets confused and misses some of the data 
                cur_col_nonan = remove_nans(continuous_col)
                
                # Edges is a list containing the left edge of every bin,
                #   hist is the frequency for every bin
                # This calls numpy.histogram
                hist, edges = histogram_func(cur_col_nonan)
                
                res_trackname = cur_res[trackname]
                
                assert (res_trackname[1] == edges).all()
                res_trackname[0] += hist
        if quick: break  # 1 chromosome
    return res

## Saves the histogram data to a tab file
def save_tab(labels, histogram, dirpath, clobber=False, group_labels=False,
             namebase=NAMEBASE, fieldnames=FIELDNAMES):
    with tab_saver(dirpath, namebase, fieldnames, clobber=clobber) as saver:
        for label_key, label_histogram in histogram.iteritems():
            label = labels[label_key]
            for trackname, (hist, edges) in label_histogram.iteritems():
                for lower_edge, count in zip(edges, hist.tolist() + ["NA"]):
                    saver.writerow(locals())
            if group_labels: return  # Only write grouped label

## Plots the tab file data and saves the plots
def save_plot(num_tracks, num_labels, segtracks, dirpath,
              clobber=False, group_labels=False,
              basename=NAMEBASE, ecdf=False, mnemonics=[]):
    """
    num_tracks: number of tracks in the Genome
    num_labels: number of labels in the Segmentation
    segtracks: a list of strings; the name of each track in the segmentation
    """
    r_source("common.R")
    r_source("signal.R")

    if group_labels:  # Only one plot per track
        num_labels = 0
        mnemonics = []  # Ignore mnemonics

    # load data from the file just written out
    # it's bad to remake the filename
    # XXX: rewriting everything in terms of a class would allow avoiding this
    tabfilename = make_tabfilename(dirpath, basename)
    if not os.path.isfile(tabfilename):
        die("Unable to find tab file: %s" % tabfilename)

    print >>sys.stderr, "\tPlotting distributions from: %s" % tabfilename
    # XXX: Figure out num_tracks and num_labels from file
    with image_saver(dirpath, basename,
                     width=PNG_BASE_WIDTH + \
                         num_tracks * PNG_WIDTH_PER_SUBPLOT,
                     height=PNG_BASE_HEIGHT + \
                         (num_labels + 1) * PNG_HEIGHT_PER_SUBPLOT,
                     clobber=clobber,
                     downsample=True):  # Speeds up plotting
        r.plot(r["plot.signal"](tabfilename, segtracks=segtracks,
                                mnemonics=mnemonics, ecdf=ecdf))

def save_html(dirpath, ecdf=False, clobber=False):
    if ecdf:
        title = "%s (ECDF mode)" % HTML_TITLE
    else:
        title = HTML_TITLE
        
    save_html_div(HTML_TEMPLATE_FILENAME, dirpath, NAMEBASE, clobber=clobber,
                  module=MODULE, title=title)
    
## Package entry point
def validate(bedfilename, genomedatadir, dirpath, group_labels=False,
             clobber=False, calc_ranges=False,
             quick=False, replot=False, noplot=False,
             num_bins=NUM_BINS, value_range=(None, None),
             ecdf=False, mnemonicfilename=None):
    
    setup_directory(dirpath)
    genome = load_genome(genomedatadir)
    segmentation = load_segmentation(bedfilename)
    
    assert genome is not None
    assert segmentation is not None
    
    mnemonics = map_mnemonics(segmentation.labels, mnemonicfilename)
    num_tracks = get_num_tracks(genome)  # All tracks (not just segtracks)
    segtracks = segmentation.tracks
    labels = segmentation.labels
    
    if group_labels:
        num_bins = None  # Set bins automatically
        num_labels = 0  # Not using labels
        fieldnames = FIELDNAMES[1:]  # No 'label' field
    else:
        num_labels = len(labels)
        fieldnames = FIELDNAMES
        
    if not replot:
        # Generate histogram and save tab and plot files
        histogram = calc_histogram(genome, segmentation, num_bins=num_bins,
                                   group_labels=group_labels,
                                   calc_ranges=calc_ranges,
                                   value_range=value_range,
                                   quick=quick)
    
        save_tab(labels, histogram, dirpath, clobber=clobber,
                 group_labels=group_labels, fieldnames=fieldnames)

    if not noplot:
        save_plot(num_tracks, num_labels, segtracks, dirpath, clobber=clobber,
                  group_labels=group_labels, ecdf=ecdf, mnemonics=mnemonics)

    save_html(dirpath, ecdf=ecdf, clobber=clobber)
                
            
def parse_options(args):
    from optparse import OptionParser, OptionGroup

    usage = "%prog [OPTIONS] BEDFILE GENOMEDATADIR"
    version = "%%prog %s" % __version__
    parser = OptionParser(usage=usage, version=version)

    group = OptionGroup(parser, "Flags")
    group.add_option("--clobber", action="store_true",
                     dest="clobber", default=False,
                     help="Overwrite existing output files if the specified"
                     " directory already exists.")
    group.add_option("--quick", action="store_true", 
                     dest="quick", default=False,
                     help="Compute values only for one chromosome.")
    group.add_option("--replot", action="store_true", 
                     dest="replot", default=False,
                     help="Load data from output tab files and"
                     " regenerate plots instead of recomputing data")
    group.add_option("--noplot", action="store_true", 
                     dest="noplot", default=False,
                     help="Do not generate plots")
    group.add_option("--group-labels", action="store_true", 
                     dest="group_labels", default=False,
                     help="Group track distributions over all labels."
                     " BEDFILE will be ignored")
    group.add_option("--ecdf", action="store_true", 
                     dest="ecdf", default=False,
                     help="Plot empiracle cumulative density inside each panel"
                     " instead of a normal histogram (turns off log-y)")
    group.add_option("--calc-ranges", action="store_true", 
                     dest="calc_ranges", default=False,
                     help="Calculate ranges for distribution plots from"
                     " segmentation data (slower) instead of using whole"
                     " genome data (default).")
    parser.add_option_group(group)
    
    group = OptionGroup(parser, "Histogram options")
    group.add_option("-n", "--num-bins", type="int",
                     dest="num_bins", default=NUM_BINS,
                     help="Number of bins for signal distribution"
                     " [default: %default]")
    group.add_option("--min-value", type="float",
                     dest="min_value", default=None,
                     help="Minimum signal track value used in binning"
                     " (overrides min from --calc-ranges)"
                     " (values below will be ignored)")
    group.add_option("--max-value", type="float",
                     dest="max_value", default=None,
                     help="Maximum signal track value used in binning"
                     " (overrides max from --calc-ranges)"
                     " (values above will be ignored)")
    parser.add_option_group(group)
    
    group = OptionGroup(parser, "I/O options")
    group.add_option("--mnemonic-file", dest="mnemonicfilename",
                     default=None,
                     help="If specified, labels will be shown using"
                     " mnemonics found in this file")
    group.add_option("-o", "--outdir", 
                     dest="outdir", default="%s" % MODULE,
                     help="File output directory (will be created"
                     " if it does not exist) [default: %default]")
    parser.add_option_group(group)

    (options, args) = parser.parse_args(args)

    if len(args) < 2:
        parser.error("Insufficient number of arguments")
        
    if options.noplot and options.replot:
        parser.error("noplot and replot are contradictory")
        
    return (options, args)
    
## Command-line entry point
def main(args=sys.argv[1:]):
    (options, args) = parse_options(args)
    bedfilename = args[0]
    genomedatadir = args[1]
    value_range = (options.min_value, options.max_value)
    validate(bedfilename, genomedatadir, options.outdir, options.group_labels,
             clobber=options.clobber, quick=options.quick,
             calc_ranges=options.calc_ranges, replot=options.replot,
             noplot=options.noplot, num_bins=options.num_bins,
             value_range=value_range, ecdf=options.ecdf, 
             mnemonicfilename=options.mnemonicfilename)

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
from __future__ import division, with_statement

__version__ = "$Revision: 90 $"

"""
seg_overlap.py:

Evaluates the overlap between two BED files, based upon the spec at:
http://encodewiki.ucsc.edu/EncodeDCC/index.php/Overlap_analysis_tool_specification


Author: Orion Buske <orion.buske@gmail.com>
Date:   August 18, 2009
"""

import math
import os
import sys

from collections import defaultdict
from math import ceil
from numpy import invert, logical_or, minimum, where, zeros
from rpy2.robjects import r, numpy2ri

# XXX: Do this without the kludgy constants
from .common import die, get_ordered_labels, image_saver, load_segmentation, make_tabfilename, map_mnemonics, maybe_gzip_open, OutputMasker, r_source, SEGMENT_START_COL, SEGMENT_END_COL, SEGMENT_LABEL_KEY_COL, setup_directory, tab_saver, template_substitute

from .bed import read_native
from .html import find_output_files, save_html_div

# A package-unique, descriptive module name used for filenames, etc
MODULE = "overlap"

NAMEBASE = "%s" % MODULE
FORWARD_NAMEBASE = os.extsep.join([NAMEBASE, "seg"])
BACKWARD_NAMEBASE = os.extsep.join([NAMEBASE, "feature"])
SIGNIFICANCE_NAMEBASE = os.extsep.join([NAMEBASE, "significance"])

HTML_TITLE_BASE = "Overlap statistics"
HTML_TEMPLATE_FILENAME = "overlap_div.tmpl"
SIGNIFICANCE_TEMPLATE_FILENAME = "overlap_significance.tmpl"

ROW_NAME_COL = ""
NONE_COL = "none"
TOTAL_COL = "total"

BY_CHOICES = ["segments", "bases"]
BY_DEFAULT = "segments"
MIDPOINT_CHOICES = ["1", "2", "both"]
SAMPLES_DEFAULT = 1000
REGION_FRACTION_DEFAULT = 0.5
SUBREGION_FRACTION_DEFAULT = 0.5

PNG_SIZE_PER_PANEL = 400  # px
SIGNIFICANCE_PNG_SIZE = 600  # px

def start_R():
    r_source("common.R")
    r_source("overlap.R")

def calc_overlap(subseg, qryseg, quick=False, by=BY_DEFAULT,
                 min_overlap=1, min_overlap_fraction=None):
    # Ensure either min_overlap or _fraction, but not both
    if min_overlap_fraction is None:
        min_overlap = int(min_overlap)
    else:
        min_overlap = None
        min_overlap_fraction = float(min_overlap_fraction)
        assert min_overlap_fraction >= 0 and min_overlap_fraction <= 1
    
    sub_labels = subseg.labels
    qry_labels = qryseg.labels
    # Make sure reserved column names aren't used
    for col_label in [ROW_NAME_COL, NONE_COL, TOTAL_COL]:
        if col_label in sub_labels.values():
            die("SUBJECTFILE uses reserved group name: %s" % col_label)
        elif col_label in qry_labels.values():
            die("QUERYFILE uses reserved group name: %s" % col_label)
            
    counts = zeros((len(sub_labels), len(qry_labels)), dtype="int")
    totals = zeros(len(sub_labels), dtype="int")
    nones = zeros(len(sub_labels), dtype="int")
    for chrom in subseg.chromosomes:
        print >>sys.stderr, "\t%s" % chrom
        try:
            qry_segments = qryseg.chromosomes[chrom]
        except KeyError:
            print >>sys.stderr, "\t\tNot in QUERYFILE. Skipping."
            continue

        # Track upper and lower bounds on range of segments that might
        #   be in the range of the current segment (to keep O(n))
        low_qry_segment_iter = iter(qry_segments)
        high_qry_segment_iter = iter(qry_segments)
        low_index = 0
        low_stop = False
        low_qry_segment = low_qry_segment_iter.next()
        high_index = 0
        high_stop = False
        high_qry_segment = high_qry_segment_iter.next()

        # Keep track of the number of segments of each group that are currently
        #   in range of the current segment
        qrygroup_counts = zeros(len(qry_labels), dtype="int")
        
        for sub_segment in subseg.chromosomes[chrom]:
            substart = sub_segment[SEGMENT_START_COL]
            subend = sub_segment[SEGMENT_END_COL]
            sublabelkey = sub_segment[SEGMENT_LABEL_KEY_COL]
            sublength = subend - substart
            # Compute min-overlap in terms of bases, if conversion required
            if min_overlap is None:
                # Be conservative with ceil
                min_overlap_bp = ceil(min_overlap_fraction * sublength)
            else:
                min_overlap_bp = min_overlap

            # High_index is just above top of range around subsegment
            while not high_stop and high_qry_segment[SEGMENT_START_COL] <= sub_segment[SEGMENT_END_COL] - min_overlap:               
                try:
                    qrygroup_counts[high_qry_segment[SEGMENT_LABEL_KEY_COL]] += 1
                    high_qry_segment = high_qry_segment_iter.next()
                    high_index += 1
                except StopIteration:
                    high_index = len(qry_segments)
                    high_stop = True
                    break
                
            # Low_index is bottom of range around subsegment
            while not low_stop and low_qry_segment[SEGMENT_END_COL] - min_overlap < sub_segment[SEGMENT_START_COL]:
                try:
                    qrygroup_counts[low_qry_segment[SEGMENT_LABEL_KEY_COL]] -= 1
                    low_qry_segment = low_qry_segment_iter.next()
                    low_index += 1
                except StopIteration:
                    low_index = len(qry_segments)
                    low_stop = True
                    break

            if by == "segments":
                full_score = 1
            elif by == "bases":
                full_score = sublength

            totals[sublabelkey] += full_score

            # Skip processing if there are no overlapping segments
            if high_index - low_index <= 0:
                nones[sublabelkey] += full_score
                continue

            # At least one segment overlaps, so calculate overlap
            range_segments = qry_segments[low_index:high_index, :]
            label_col = range_segments[:, SEGMENT_LABEL_KEY_COL]
                
            if by == "segments":
                # Add 1 to count for each group currently in range
                groups_in_range = (qrygroup_counts > 0)
                assert groups_in_range.sum() != 0
                counts[sublabelkey] += groups_in_range
 
            elif by == "bases":
                # Keep track of total covered by all labels
                covered = zeros(sublength, dtype="bool")
                for qrylabelkey in qry_labels:
                    # Look at total covered by single label
                    label_rows = (label_col == qrylabelkey)
                    label_covered = zeros(sublength, dtype="bool")
                    for qry_segment in range_segments[label_rows, :]:
                        qrystart = qry_segment[SEGMENT_START_COL]
                        qryend = qry_segment[SEGMENT_END_COL]
                        # Define bounds of coverage
                        cov_start = min(qrystart - substart, 0)
                        cov_end = max(cov_start + (qryend - qrystart),
                                      sublength)
                        label_covered[cov_start:cov_end]=True
                        
                    # Add the number of bases covered by this segment
                    counts[sublabelkey][qrylabelkey] += label_covered.sum()
                    covered = logical_or(covered, label_covered)

                # See how many bases were never covered by any segment
                nones[sublabelkey] += invert(covered).sum()

        if quick: break
    return (counts, totals, nones)

def split_bed_regions(filename, labels, lengths):
    """Simultaneously splits a bed file into region objects for each label.

    Mostly a direct copy of code in Nathan Boley's GSC
    (encode.py:parse_bed_file)
    Should be ~25 times faster than re-parsing with the 'group' arg for each
    label
    """
    from encode import binary_region, interval, regions

    # Initialize dict of regions
    chrom_dict = {}
    for label in labels.values():
        chrom_dict[label] = regions()
        for chName in lengths.keys():
            length = lengths[chName].length
            chrom_dict[label][chName] = binary_region((), chName, length)

    with maybe_gzip_open(filename) as infile:
        for datum in read_native(infile):
            feature = interval(int(datum.chromStart), int(datum.chromEnd))
            chName = datum.chrom
            name = datum.name
            if not lengths.has_key( chName ):
                continue
            
            shifted_feature = lengths[chName].intersection(feature) 
            # if there is no intersection, there is nothing else to do
            if shifted_feature == None: continue
            else: 
                try:
                    chrom_dict[name][chName].add(shifted_feature)
                # This assumes that, if the region name is not present
                # in the lengths file, then we want to ignore it
                except KeyError:
                    pass
                
    return chrom_dict

def parse_file_regions(parser, filename, labels, lengths):
    regions = {}
    for key, label in labels.iteritems():
        print >>sys.stderr, "\t%s" % label
        with maybe_gzip_open(filename) as file:
            regions[key] = parser(file, lengths,
                                  group=label,
                                  force_binary=True)
    return regions

def calc_significance(seg_labels, feature_labels, bedfilename,
                      featurefilename, regionfilename,
                      num_samples=SAMPLES_DEFAULT,
                      region_fraction=REGION_FRACTION_DEFAULT,
                      subregion_fraction=SUBREGION_FRACTION_DEFAULT,
                      quick=False, by=BY_DEFAULT):
    """Use Genome Structure Correction tool to measure overlap significance.

    If GSC is not in the user's PYTHONPATH or the user failed to provide a
    region file for GSC, skip this part of validation and move on.
    
    Returns a nested dictionary mapping label-feature pairs to p_values:
      dict ( seg_label_key -> dict ( feature_label_key -> p_value ) )
    """
    try:
        import block_bootstrap
    except ImportError:
        print >> sys.stderr, "GSC not on PYTHONPATH. Skipping significance statistics."
        return None
    else:
        print >>sys.stderr, "Found GSC"

    from block_bootstrap import marginal_bp_overlap_stat, marginal_resample_region_overlap_stat, output
    block_bootstrap.output = OutputMasker()  # Block GSC stdout output
    from encode import parse_bed_file, parse_lengths_file
        
    if regionfilename is None:
        print >> sys.stderr, "A region file must be specified to use GSC. Skipping significance statistics."
        return None
    elif not os.path.isfile(regionfilename):
        print >> sys.stderr, "Could not locate region file: %s. Skipping significance statistics." % regionfilename
        return None
    else:
        print >>sys.stderr, "Using region file: %s" % regionfilename

    # Parse region file
    print >>sys.stderr, "Loading region file...",
    sys.stdout.flush()
    with maybe_gzip_open(regionfilename) as regionfile:
        lengths = parse_lengths_file(regionfile)
    print >>sys.stderr, "done"

    # Preprocess both input BED files, splitting by labels
    
    # Parse segmentation BED file, once for each label
    print >>sys.stderr, "Parsing segmentation by label...",
    sys.stdout.flush()
    seg_regions = split_bed_regions(bedfilename, seg_labels, lengths)
    #seg_regions = parse_file_regions(parse_bed_file, bedfilename,
    #                                 seg_labels, lengths)
    print >>sys.stderr, "done"
    
    # Parse feature BED file, once for each label
    print >>sys.stderr, "Parsing features by label...",
    sys.stdout.flush()
    feature_regions = split_bed_regions(featurefilename,
                                        feature_labels, lengths)
    #feature_regions = parse_file_regions(parse_bed_file, featurefilename,
    #                                     feature_labels, lengths)
    print >>sys.stderr, "done"

    # Calculate p-value for every label-feature pair
    p_values = {}
    print >>sys.stderr, "Calculating p-values:"
    for seg_label_key, seg_label in seg_labels.iteritems():
        print >>sys.stderr, "\t%s" % seg_label
        p_values[seg_label_key] = {}
        for feature_label_key, feature_label in feature_labels.iteritems():
            # Use GSC to calculate p-value
            try:
                sys.stdout = OutputMasker(sys.stdout)  # Kill stdout prints
                if by == "segments":
                    z, p = marginal_resample_region_overlap_stat(
                        seg_regions[seg_label], feature_regions[feature_label],
                        region_fraction, subregion_fraction, num_samples)
                elif by == "bases":
                    z, p = marginal_bp_overlap_stat(
                        seg_regions[seg_label], feature_regions[feature_label], 
                        region_fraction, num_samples)
                else:
                    print >> sys.stderr, "Unrecognized mode: %s. Skipping significance testing." % by
                    return None
                    
            except Exception, e:
                print >> sys.stderr, "GSC raised the following exception: %s.\nSkipping significance testing." % e
                return None
            finally:
                sys.stdout = sys.stdout.restore()
                
            p_values[seg_label_key][feature_label_key] = p

        if quick: break

    print(str(p_values))
    return p_values
    
def make_tab_row(row_labels, row_key, col_labels, col_ordered_keys,
                 data, nones=None, totals=None, formatstr="%d"):
    row = {ROW_NAME_COL : row_labels[row_key]}
    for col_key in col_ordered_keys:
        row[col_labels[col_key]] = formatstr % data[row_key][col_key]
            
    if nones is not None:
        row[NONE_COL] = nones[row_key]

    if totals is not None:
        row[TOTAL_COL] = totals[row_key]
        
    return row

## Saves the data to a tab file
def save_tab(dirpath, row_labels, col_labels, counts, totals, nones,
             namebase=NAMEBASE, clobber=False,
             mnemonic_rows=[], mnemonic_cols=[]):
    assert counts is not None and totals is not None and nones is not None
    
    row_ordered_keys, row_labels = get_ordered_labels(row_labels, mnemonic_rows)
    col_ordered_keys, col_labels = get_ordered_labels(col_labels, mnemonic_cols)

    # Set up fieldnames based upon QUERYFILE groups
    fieldnames = list(col_labels[col_key] for col_key in col_ordered_keys)
    fieldnames.insert(0, ROW_NAME_COL)
    fieldnames.append(NONE_COL)
    fieldnames.append(TOTAL_COL)
    with tab_saver(dirpath, namebase, fieldnames,
                   clobber=clobber) as count_saver:
        for row_key in row_ordered_keys:
            count_row = make_tab_row(row_labels, row_key, col_labels,
                                     col_ordered_keys, counts,
                                     nones, totals)
            count_saver.writerow(count_row)
            
## Saves the significance data to a tab file
def save_significance_tab(dirpath, seg_labels, feature_labels, p_values, 
             namebase=SIGNIFICANCE_NAMEBASE, clobber=False,
             mnemonics=[]):
    assert p_values is not None
    
    row_ordered_keys, row_labels = get_ordered_labels(seg_labels, mnemonics)
    col_ordered_keys, col_labels = get_ordered_labels(feature_labels)

    # Set up fieldnames based upon feature groups
    fieldnames = list(col_labels[col_key] for col_key in col_ordered_keys)
    fieldnames.insert(0, ROW_NAME_COL)
    with tab_saver(dirpath, namebase, fieldnames,
                   clobber=clobber) as count_saver:
        for row_key in row_ordered_keys:
            if row_key in p_values:
                count_row = make_tab_row(row_labels, row_key, col_labels,
                                         col_ordered_keys, p_values,
                                         formatstr="%.0e")
                count_saver.writerow(count_row)

def save_plot(dirpath, num_panels, clobber=False, mnemonics=[]):
    start_R()

    tabfilename = make_tabfilename(dirpath, FORWARD_NAMEBASE)
    if not os.path.isfile(tabfilename):
        die("Unable to find tab file: %s" % tabfilename)

    panels_sqrt = math.sqrt(num_panels)
    width = math.ceil(panels_sqrt)
    height = math.floor(panels_sqrt)
        
    # Plot data in file
    with image_saver(dirpath, NAMEBASE, clobber=clobber,
                     width=PNG_SIZE_PER_PANEL * width,
                     height=PNG_SIZE_PER_PANEL * height):
        r.plot(r["plot.overlap"](tabfilename,
                                 dirpath=dirpath,
                                 basename=NAMEBASE,
                                 mnemonics=mnemonics))
        
def save_significance_plot(dirpath, clobber=False, mnemonics=[]):
    start_R()
    
    tabfilename = make_tabfilename(dirpath, SIGNIFICANCE_NAMEBASE)
    if not os.path.isfile(tabfilename):
        print >> sys.stderr, "Unable to find tab file: %s. Skipping significance plot." % tabfilename
        return
        
    # Plot data in file
    with image_saver(dirpath, SIGNIFICANCE_NAMEBASE, clobber=clobber,
                     width=SIGNIFICANCE_PNG_SIZE,
                     height=SIGNIFICANCE_PNG_SIZE):
        r.plot(r["plot.overlap.pvalues"](tabfilename, mnemonics=mnemonics))
                
def save_html(dirpath, bedfilename, featurefilename, by, clobber=False):
    bedfilename = os.path.basename(bedfilename)
    featurebasename = os.path.basename(featurefilename)

    title = "%s (%s)" % (HTML_TITLE_BASE, featurebasename)
    
    # Insert significance results if they were made
    tabfilename = make_tabfilename(dirpath, SIGNIFICANCE_NAMEBASE)
    if os.path.isfile(tabfilename):
        files = find_output_files(dirpath, SIGNIFICANCE_NAMEBASE)
        significance = template_substitute(
            SIGNIFICANCE_TEMPLATE_FILENAME)(files)
    else:
        significance = ""
        
    save_html_div(HTML_TEMPLATE_FILENAME, dirpath, NAMEBASE, clobber=clobber,
                  title=title, forwardtablenamebase=FORWARD_NAMEBASE,
                  backwardtablenamebase=BACKWARD_NAMEBASE,
                  module=MODULE, by=by, significance=significance,
                  bedfilename=bedfilename, featurefilename=featurebasename)

## Package entry point
def validate(bedfilename, featurefilename, dirpath, regionfilename=None,
             clobber=False, quick=False,
             by=BY_DEFAULT, samples=SAMPLES_DEFAULT,
             region_fraction=REGION_FRACTION_DEFAULT,
             subregion_fraction=SUBREGION_FRACTION_DEFAULT,
             min_overlap=1, min_overlap_fraction=None,
             mnemonicfilename=None, replot=False, noplot=False):
    setup_directory(dirpath)
    segmentation = load_segmentation(bedfilename, checknames=False)
    features = load_segmentation(featurefilename, checknames=False)
    
    assert segmentation is not None
    assert features is not None

    seg_labels = segmentation.labels
    feature_labels = features.labels
    mnemonics = map_mnemonics(seg_labels, mnemonicfilename)
    
    if not replot:
        # Overlap of segmentation with features (forward)
        print >>sys.stderr, "Measuring overlap of segmentation with features..."
        seg_counts, seg_totals, seg_nones = calc_overlap(
            segmentation, features, by=by, min_overlap=min_overlap,
            min_overlap_fraction=min_overlap_fraction, quick=quick)
        save_tab(dirpath, seg_labels, feature_labels, seg_counts,
                 seg_totals, seg_nones, namebase=FORWARD_NAMEBASE,
                 clobber=clobber, mnemonic_rows=mnemonics)  # Labels on rows
        
        # Overlap of features with segmentation (backward)
        print >>sys.stderr, "Measuring overlap of features with segmentation..."
        feature_counts, feature_totals, feature_nones=calc_overlap(
            features, segmentation, by=by, min_overlap=min_overlap,
            min_overlap_fraction=min_overlap_fraction, quick=quick)
        save_tab(dirpath, feature_labels, seg_labels, feature_counts,
                 feature_totals, feature_nones, namebase=BACKWARD_NAMEBASE,
                 clobber=clobber, mnemonic_cols=mnemonics)  # Labels on cols

        # GSC significance of forward overlap
        print >>sys.stderr, "Measuring significance of overlap of segmentation with features..."
        p_values = calc_significance(seg_labels, feature_labels, bedfilename,
                                     featurefilename, regionfilename,
                                     num_samples=samples,
                                     region_fraction=region_fraction,
                                     subregion_fraction=subregion_fraction,
                                     quick=quick, by=by)
        if p_values is not None:
            save_significance_tab(dirpath, seg_labels, feature_labels,
                                  p_values=p_values, clobber=clobber,
                                  mnemonics=mnemonics)

    if not noplot:
        save_plot(dirpath, num_panels=len(feature_labels),
                  clobber=clobber, mnemonics=mnemonics)
        save_significance_plot(dirpath, clobber=clobber,
                               mnemonics=mnemonics)
        
    print >>sys.stderr, "Saving HTML div...",
    sys.stdout.flush()  # Necessary to make sure html errors don't clobber print
    save_html(dirpath, bedfilename, featurefilename, by=by, clobber=clobber)
    print >>sys.stderr, "done"
    
def parse_options(args):
    from optparse import OptionParser, OptionGroup

    usage = "%prog [OPTIONS] BEDFILE FEATUREFILE"
    description = "BEDFILE and FEATUREFILE should both be in BED3+ format (gzip'd okay). BEDFILE should correspond to a segmentation. Overlap analysis will be performed in both directions (BEDFILE as SUBJECTFILE and QUERYFILE). See for full specification: http://encodewiki.ucsc.edu/EncodeDCC/index.php/Overlap_analysis_tool_specification"
    
    version = "%%prog %s" % __version__
    parser = OptionParser(usage=usage, version=version, description=description)

    group = OptionGroup(parser, "Flags")
    group.add_option("--clobber", action="store_true",
                     dest="clobber", default=False,
                     help="Overwrite existing output files if the specified"
                     " directory already exists.")
    group.add_option("--quick", action="store_true", 
                     dest="quick", default=False,
                     help="Compute values only for one chromosome.")
    group.add_option("--replot", action="store_true", 
                     dest="replot", default=False,
                     help="Load data from output tab files and"
                     " regenerate plots instead of recomputing data")
    group.add_option("--noplot", action="store_true", 
                     dest="noplot", default=False,
                     help="Do not generate plots")
    parser.add_option_group(group)
    
    group = OptionGroup(parser, "Parameters")
    group.add_option("-b", "--by", choices=BY_CHOICES,
                     dest="by", type="choice", default=BY_DEFAULT,
                     help="One of: "+str(BY_CHOICES)+", which determines the"
                     " definition of overlap. @segments: The value"
                     " associated with two features overlapping will be 1 if"
                     " they overlap, and 0 otherwise. @bases: The value"
                     " associated with two features overlapping will be"
                     " number of base pairs which they overlap."
                     " [default: %default]")
    group.add_option("--midpoint-only", choices=MIDPOINT_CHOICES,
                     dest="midpoint", type="choice", default=None,
                     help="For the specified file (1, 2, or both), use only"
                     "the midpoint of each feature instead of the entire"
                     " width.")
    group.add_option("-m", "--min-overlap", type="int",
                     dest="min_overlap", default=1,
                     help="The minimum number of base pairs that two"
                     " features must overlap for them to be classified as"
                     " overlapping. This integer can be either positive"
                     " (features overlap only if they share at least this"
                     " many bases) or negative (features overlap if there"
                     " are no more than this many bases between them). Both"
                     " a negative min-overlap and --by=bases cannot be"
                     " specified together. [default: %default]")
    group.add_option("--min-overlap-fraction", type="float",
                     dest="min_overlap_fraction", default=None,
                     help="The minimum fraction of the base pairs in the"
                     " subject feature that overlap with the query feature"
                     " in order to be counted as overlapping. Overrides"
                     "--min-overlap.")
    parser.add_option_group(group)

    group = OptionGroup(parser, "Files")
    group.add_option("--mnemonic-file", dest="mnemonicfilename",
                     default=None,
                     help="If specified, labels will be shown using"
                     " mnemonics found in this file")
    group.add_option("-o", "--outdir", 
                     dest="outdir", default="%s" % MODULE,
                     help="File output directory (will be created"
                     " if it does not exist) [default: %default]")
    parser.add_option_group(group)

    group = OptionGroup(parser, "GSC Options")
    group.add_option("--region-file", dest="regionfilename",
                     default=None,
                     help="If specified, this file will be used to calculate"
                     " overlap significance using GSC. This must be a BED file")
    group.add_option("-s", "--samples", type="int",
                     dest="samples", default=SAMPLES_DEFAULT,
                     help="The number of samples for GSC to use to estimate"
                     " the significance of the overlap [default: %default]")
    group.add_option("--region-fraction", type="float",
                     dest="region_fraction",
                     default=REGION_FRACTION_DEFAULT,
                     help="The region_fraction tu use with GSC"
                     " [default: %default]")
    group.add_option("--subregion-fraction", type="float",
                     dest="subregion_fraction",
                     default=SUBREGION_FRACTION_DEFAULT,
                     help="The subregion_fraction tu use with GSC"
                     " [default: %default]")
    parser.add_option_group(group)

    (options, args) = parser.parse_args(args)

    if len(args) < 2:
        parser.error("Insufficient number of arguments")

    mof = options.min_overlap_fraction
    if mof is not None and (mof < 0 or mof > 1):
        parser.error("Min-overlap-fraction: %.3f out of range: [0, 1]" % mof)
                       
    return (options, args)
    
## Command-line entry point
def main(args=sys.argv[1:]):
    (options, args) = parse_options(args)
    bedfilename = args[0]
    featurefilename = args[1]

    validate(bedfilename, featurefilename, options.outdir,
             regionfilename=options.regionfilename, clobber=options.clobber,
             quick=options.quick, by=options.by,
             min_overlap=options.min_overlap, samples=options.samples,
             min_overlap_fraction=options.min_overlap_fraction,
             region_fraction=options.region_fraction,
             subregion_fraction=options.subregion_fraction,
             mnemonicfilename=options.mnemonicfilename, 
             replot=options.replot, noplot=options.noplot)
        
if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
"""
install.py

General installation script for the segtools package. Interacts with
the user to configure the environment to download and install all
dependencies and the segtools package.

This script is designed to be downloaded and run independently, and
will guide the process of downloading and installing all other source
code.

(c) 2009: Orion Buske <orion.buske@gmail.com>

XXX: R and necessary R packages are not checked/installed.
XXX: Relatively untested
"""

####################### BEGIN COMMON CODE HEADER #####################

import os
import sys

from distutils.spawn import find_executable
from distutils.version import StrictVersion
from urllib import urlretrieve
from string import Template
from subprocess import PIPE, Popen

assert sys.version_info >= (2, 4)

MIN_HDF5_VERSION = "1.8"
MIN_NUMPY_VERSION = "1.2"

# Should match download URLs below (displayed to user)
HDF5_DOWNLOAD_VERSION = "1.8.2"

HDF5_URL = "http://www.hdfgroup.org/ftp/HDF5/prev-releases/hdf5-1.8.2/src/hdf5-1.8.2.tar.gz"
EZ_SETUP_URL = "http://peak.telecommunity.com/dist/ez_setup.py"

EASY_INSTALL_PROMPT = "May I install %s and dependencies?"
HDF5_INSTALL_MESSAGE = "\nHDF5 is very large and installation usually takes 5-10 minutes. Please be patient.\nIt is common to see many warnings during compilation."

# Template for cfg file contents
CFG_FILE_TEMPLATE = """
[install]
prefix = $prefix
exec_prefix = $prefix
install_platlib = $platlib
install_purelib = $platlib
install_scripts = $scripts

[easy_install]
install_dir = $platlib
script_dir = $scripts
"""

# One command per line
HDF5_INSTALL_SCRIPT = """
if [ ! -e $file ]; then wget $url -O $file; fi
if [ ! -d $filebase ]; then tar -xzf $file; fi
cd $filebase
./configure --prefix=$dir
make
make install
rm -f $file
"""

####################### END COMMON CODE HEADER #####################


# One command per line
SEGTOOLS_INSTALL_SCRIPT = """
cd $dir
svn co $url
cd $file
python setup.py install
"""

####################### BEGIN COMMON CODE BODY #####################

############################ CLASSES ##########################
class DependencyError(Exception):
    pass

class InstallationError(Exception):
    pass

class ShellManager(object):
    """Class to manage details of shells.

    Provides an interface for writing to shell rc files and
    setting and testing environment variables.

    Attributes:
    file: The shell rc file, or None if none known
    has_file: True if there is a known shell rc file
    name: The string name of the shell (e.g. "bash", "csh", etc.)
      or "" if unknown
    """
    def __init__(self, shell=None):
        """Creates a ShellManager given a shell string.

        If shell is None, tries to use environment variable SHELL.
        self.name: basically same as shell, but if shell is None, self.name = ""
        """
        self.name = shell
        if shell is None:
            try:
                shell = os.path.basename(os.environ["SHELL"])
                self.name = shell
            except KeyError:
                self.name = ""
                
        if self.name.endswith("csh"):
            self._env_format = "setenv %s %s"
        elif self.name.endswith("sh"):
            self._env_format = "export %s=%s"
        else:
            self._env_format = "set %s to %s"  # Console output
            
        # What RC file are we using, or are we printing to the terminal?
        if self.name.endswith("sh"):
            self.file = os.path.join("~", ".%src" % shell)
        else:
            self.file = None  # Print to terminal
            if shell is None:
                print >>sys.stderr, "SHELL variable missing or confusing."
            else:
                print >>sys.stderr, "Unrecognized shell: %s" % shell
            
        if self.file is None:
            print >>sys.stderr, ("Shell-specific commands will be printed"
                                 " to the terminal")
            
        self._out = None  # Output file not yet open
        self.has_file = (self.file is not None)
        
    def set_rc_env(self, variable, value):
        """Write the given variable and value the the shell rc file (or stdout)
        """
        if self._out is None:
            if self.file is None:
                self._out = sys.stdout
            else:
                self._out = open(os.path.expanduser(self.file), "a")
        
        cmd = self._env_format % (variable, value)
        print >>self._out, "\n%s  # Added in segway installation\n" % cmd
            
    def add_to_rc_env(self, variable, value):
        """Prepend the value to the variable in the shell rc file (or stdout)
        """
        full_value = "%s:$%s" % (value, variable)
        self.set_rc_env(variable, full_value)

    def set_env(self, variable, value):
        """Set the environment variable to the given value.
        """
        os.environ[variable] = value

    def add_to_env(self, variable, value):
        """Prepend value to the value of the environment variable.
        """
        if variable in os.environ:
            value = "%s:%s" % (value, os.environ[variable])

        self.set_env(variable, value)

    def in_env(self, variable, value):
        """Checks if value is in environment variable.

        Returns true if variable found and value is one of the ':'-
        delimited values in it.
        """
        try:
            env = os.environ[variable]
            env_values = env.strip().split(":")
            return value in env_values
        except KeyError:
            return False

    def close(self):
        """Close the open rc file, if one exists
        """
        if self._out not in [None, sys.stdout, sys.stderr]:
            try:
                self._out.close()
            except ValueError:
                pass

######################## UTIL FUNCTIONS ####################
def setup_arch_home():
    query = "\nWhere should platform-specific files be installed?"
    default_arch_home = get_default_arch_home()
    arch_home = prompt_user(query, default_arch_home)
    make_dir(arch_home)
    return arch_home
            
def get_default_arch_home():
    if "ARCHHOME" in os.environ:
        return os.environ["ARCHHOME"]
    elif "ARCH" in os.environ:
        arch = os.environ["ARCH"]
    else:
        (sysname, nodename, release, version, machine) = os.uname()
        arch = "-".join([sysname, machine])
        
    return os.path.expanduser("~/arch/%s" % arch)

def setup_python_home(arch_home = None):
    if arch_home is None:
        arch_home = get_default_arch_home()
        
    query = "\nWhere should new Python packages be installed?"
    default_python_home = get_default_python_home(arch_home)
    python_home = fix_path(prompt_user(query, default_python_home))
    make_dir(python_home)
    return python_home, default_python_home

def get_default_python_home(arch_home):
    python_version = sys.version[:3]
    return os.path.join(arch_home, "lib", "python%s" % python_version)

def setup_script_home(arch_home = None):
    if arch_home is None:
        arch_home = get_default_arch_home()
        
    query = "\nWhere should new scripts and executables be installed?"
    default_script_home = os.path.join(arch_home, "bin")
    script_home = fix_path(prompt_user(query, default_script_home))
    make_dir(script_home)
    return script_home, default_script_home

def prompt_add_to_env(shell, variable, value):
    if shell.in_env(variable, value):
        print >>sys.stderr, "\nFound %s already in your %s!" % \
            (value, variable)
    else:
        shell.add_to_env(variable, value)
        query = "\nMay I edit your %s to add %s to your %s?" % \
            (shell.file, value, variable) 
        permission = prompt_yes_no(query)
        if permission:
            shell.add_to_rc_env(variable, value)
        
def prompt_set_env(shell, variable, value):
    shell.set_env(variable, value)
    query = "\nMay I edit your %s to set your %s to %s?" % \
        (shell.file, variable, value) 
    permission = prompt_yes_no(query)
    if permission:
        shell.set_rc_env(variable, value)

def fix_path(path):
    # Put path in standard form
    return os.path.abspath(os.path.expanduser(path))
        
def make_dir(dirname, verbose=True):
    """Make directory if it does not exist"""
    absdir = fix_path(dirname)
    if not os.path.isdir(absdir):
        os.makedirs(absdir)
        if verbose:
            print >>sys.stderr, "Created directory: %s" % dirname

def substitute_template(template, fields):
    return Template(template).substitute(fields)

def check_executable_in_path(bin):
    """Checks if an executable of the given name is in the user's path.

    If found, returns the path.
    If not found (and the user chooses to continue anyway, returns None)
    """
    path = find_executable(bin)
    if not path:
        print >>sys.stderr, "Required executable: %s not found in path." % bin
        query = "Would you like to continue the installation anyway?"
        continue_inst = prompt_yes_no(query)
        if not continue_inst:
            die("\n============ Installation Aborted ============")
            
    return path

def has_lsf():
    return "LSF_ENVDIR" in os.environ

def has_sge():
    return "SGE_ROOT" in os.environ

def can_find_library(libname):
    """Returns a boolean indicating if the given library could be found"""
    try:
        from ctypes import CDLL
        CDLL(libname)
        return True
    except OSError:
        return None

########################## CFG FILE ##########################
def prompt_create_cfg(arch_home, python_home, default_python_home,
                      script_home, default_script_home,
                      cfg_file="~/.pydistutils.cfg"):
    cfg_path = fix_path(cfg_file)
    if os.path.isfile(cfg_path):
        print >>sys.stderr, ("\nFound your %s! (hopefully the configuration"
                             " matches)") % cfg_file
    else:
        query = ("\nMay I create %s? It will be used by distutils"
                 " to install new Python modules into this directory"
                 " (and subdirectories) automatically.") % cfg_file
        permission = prompt_yes_no(query)
        if permission:
            write_pydistutils_cfg(cfg_path, arch_home,
                                  python_home, default_python_home,
                                  script_home, default_script_home)
    
def write_pydistutils_cfg(cfg_file, arch_home,
                          python_home, default_python_home,
                          script_home, default_script_home):
    """Write a pydistutils.cfg file
    """
    fields = {}
    fields["prefix"] = arch_home
    
    if python_home == default_python_home:
        platlib = "$platbase/lib/python$py_version_short"
    else:
        platlib = python_home
    fields["platlib"] = platlib

    if script_home == default_script_home:
        scripts = "$platbase/bin"
    else:
        scripts = script_home
    fields["scripts"] = scripts

    cfg_file_contents = substitute_template(CFG_FILE_TEMPLATE, fields)

    ofp = open(cfg_file, "w")
    try:
        print >>ofp, cfg_file_contents
    finally:
        ofp.close()
    
    
########################### GET VERSION ########################
def get_hdf5_version():
    """Returns HDF5 version as string or None if not found or installed

    Only works if h52gif is installed and in current user path
    """
    try:
        cmd = Popen("h52gif -V", shell=True, stdout=PIPE, stderr=PIPE)
        res = cmd.stdout.readlines()[0].strip()
        if "Version" in res:
            # HDF5 Found!
            return res.split("Version ")[1]
        else:
            return None
    except (OSError, IndexError):
        return None

def get_numpy_version():
    """Returns Numpy version as a string or None if not found or installed
    """
    try:
        import numpy
        return numpy.__version__
    except (AttributeError, ImportError):
        return None

def get_setuptools_version():
    """Returns setuptools version as a string or None if not found or installed
    """
    try:
        import setuptools
        return setuptools.__version__
    except (AttributeError, ImportError):
        return None
    
def str2version(ver):  # string to version object
    if ver.startswith("$Revision:"):
        ver = ver.split()[1]  # Get revision number
    return StrictVersion(ver)

##################### SPECIFIC PROGRAM INSTALLERS ################
def prompt_install_hdf5(arch_home):
    return _installer("HDF5", install_hdf5, get_hdf5_version,
                      install_message=HDF5_INSTALL_MESSAGE, arch_home=arch_home,
                      version=HDF5_DOWNLOAD_VERSION)

def prompt_install_numpy():
    return _installer("Numpy", install_numpy, get_numpy_version,
                      min_version=MIN_NUMPY_VERSION,
                      install_prompt=EASY_INSTALL_PROMPT)

def install_hdf5(arch_home, *args, **kwargs):
    hdf5_dir = prompt_install_path("HDF5", arch_home)
    install_dir = install_script("HDF5", hdf5_dir, HDF5_INSTALL_SCRIPT,
                                 url=HDF5_URL)
    return install_dir

def install_numpy(min_version=MIN_NUMPY_VERSION, *args, **kwargs):
    # Unset LDFLAGS when installing numpy as kludgy solution to
    #   http://projects.scipy.org/numpy/ticket/182
    env_old = None
    if "LDFLAGS" in os.environ:
        env_old = os.environ["LDFLAGS"]
        del os.environ["LDFLAGS"]

    try:
        return easy_install("numpy", min_version=min_version)
    finally:
        if env_old is not None:
            # Make sure variable didn't return, and then replace variable
            assert "LDFLAGS" not in os.environ
            os.environ["LDFLAGS"] = env_old
    
#################### GENERIC INSTALL METHODS ###################         
def _installer(progname, install_func, version_func=None,
               install_message=None, *args, **kwargs):
    """Program installation wrapper method.

    Input:
    progname: string name of program
    install_func: function to call with *args, **kwargs to install progname
    version_func: function to call with *args, **kwargs to find progname version
    install_message: helpful message to print if user chooses to install program

    Checks if program is installed in a succificent version,
    if not, prompts user and installs program.

    Returns result of installation if installation attempted,
    else returns False
    """
    found = _check_install(progname, version_func, *args, **kwargs)
    if not found:
        permission = prompt_install(progname, *args, **kwargs)
        if permission:
            if install_message: print str(install_message)
            success = _abort_skip_install(install_func, *args, **kwargs)
            if success:
                print >>sys.stderr, "%s successfully installed." % progname
            else:
                print >>sys.stderr, "%s not installed." % progname

            return success
        
    return False

def _abort_skip_install(func, *args, **kwargs):
    """Calls func with args. If it fails, prompts user to abort or skip.

    On success, returns what the function would.
    On failure and skip, returns None
    On failure and abort, program terminates
    """
    try:
        return func(*args, **kwargs)
    except InstallationError:
        query = ("\nWould you like to try to continue the installation"
                 " without this program?")
        default = "n"
        permission = prompt_yes_no(query, default)
        if permission:
            return None
        else:
            die("\n============== Installation aborted =================")
            
def _check_install(progname, version_func, min_version=None, *args, **kwargs):
    """Returns True if program found with at least min_version, False otherwise

    version_func should be a function that, when called, returns the version of
    the installation as a tuple, or True if installed,
    or None if not found/unavailable.

    If version_func returns True, installation accepted regardless of
    min_version
    """
    print >>sys.stdout, "\nSearching for %s..." % progname,
    sys.stdout.flush()
    version = version_func()
    if version is not None:
        print >>sys.stderr, "found!"
        if min_version is None or version is True:
            # "True" testing necessary to distinguish from tuple
            return True
        elif str2version(min_version) > str2version(version):
            print >>sys.stderr, ("Found version: %s. Version %s or above"
                                 " required.") % (version, min_version)
        else:
            return True
        
    else:
        print >>sys.stderr, "not found."

    return False

def easy_install(progname, min_version=None):
    """Easy-installs the given program (can specifiy minimum version).
    """
    if get_setuptools_version() == None:
        query = ("Unable to find setuptools. This is necessary to install"
                 " this program and many of its dependencies. May I"
                 " download and install %s?")
        permission = prompt_install("setuptools", install_prompt=query)
        if permission:
            try:
                urlretrieve(EZ_SETUP_URL)
                from ez_setup import use_setuptools
                use_setuptools(delay=0)
            except:
                print >>sys.stderr, ("Error occurred while trying to"
                                     " install setuptools")
                raise InstallationError()
        else:
            raise InstallationError()
    
    cmd = "easy_install %s" % progname.lower()
    if min_version is not None:  # Add version requirement
        cmd += ' "%s>=%s"' % (progname, min_version)

    if os.path.isdir(progname):
        print >>sys.stderr, ("\nWarning: installation may fail because"
                             " there is a subdirectory named %s at your"
                             " current path.") % progname

    print >>sys.stderr, ">> %s" % cmd
    proc = Popen(cmd, shell=True, stdout=None, stderr=None)
    code = proc.wait()

    if code != 0:
        print >>sys.stderr, "Error occured installing %s" % progname
        raise InstallationError()
    else:
        return True

def install_script(progname, prog_dir, script, **kwargs):
    """Tries to install the specified program, given a script, url

    progname: string name of program being installed
    
    prog_dir: directory program is to be installed in
    
    script: multi-line string, where each line is a command to run in the
    shell in order to install the program. Variables in this script
    will be substituted with keywords in kwargs, as well as dir
    (the installation directory) and, if a url is specified,
    the downloaded file and filebase
    
    Returns installation directory if installation is successful
    (or True if unknown), and None otherwise.
    """
    fields = kwargs

    # Set fields for template substitution
    if "dir" not in fields:
        fields["dir"] = prog_dir
        
    if "url" in fields:
        filename = os.path.basename(fields["url"])
        if "file" not in fields:
            fields["file"] = filename
        if "filebase" not in fields:
            if filename.endswith(".tar.gz"):
                fields["filebase"] = filename[:-7]  # minus .tar.gz
            else:
                fields["filebase"] = filename
    
    script = substitute_template(script, fields).strip().splitlines()

    cwd = os.getcwd()  # Keep track of cwd state

    if prog_dir is not None:
        os.chdir(prog_dir)  # Move to installation directory
        
    for line in script:
        line = line.strip()
        print >>sys.stderr, ">> %s" % line
        # Intercept cd lines and update directory state
        if line.startswith("cd"):
            try:
                path = line.split()[1]
                os.chdir(fix_path(path))
                continue
            except IndexError:
                die("Invalid cd command: %s" % line)
                
        # Run command in shell
        proc = Popen(line, shell=True, stdout=None, stderr=None)
        code = proc.wait()
        if code != 0:
            print >>sys.stderr, "Error installing: %s" % progname
            print >>sys.stderr, "Command failed: %s" % line
            os.chdir(cwd)
            raise InstallationError()

    os.chdir(cwd)
    return prog_dir

def reinstall_program(progname, *args, **kwargs):
    """Reinstall program given name and arguments."""
    progname = progname.lower()
    if progname == "hdf5":
        install_hdf5(*args, **kwargs)
    elif progname == "numpy":
        install_numpy(*args, **kwargs)
    elif progname == "segway":
        install_segway(*args, **kwargs)

########################## PROGRAM TESTING ######################
def prompt_test_packages(*args, **kwargs):
    """Run each dependency's unit tests and if they fail, prompt reinstall
    
    XXX: implement this for more than pytables (but numpy always fails)
    """
    print >>sys.stderr, "\n\n"
    try:
        prompt_test_pytables(*args, **kwargs)
        print >>sys.stderr, "\n=========== All tests passed! ============"
    except InstallationError:
        die("\n=========== Some tests failed! =============="
            "\nYour installation may be incomplete and might not work.")

def prompt_test_pytables(*args, **kwargs):
    query = ("May I test the PyTables installation? This should provide"
             " a reasonable test of the HDF5 and NumPy installations as well.")
    permission = prompt_yes_no(query)
    if permission:
        try:
            import tables
            tables.test()
            print >>sys.stderr, "Test passed!"
        except:
            print >>sys.stderr, ("There seems to be an error with the"
                                 " PyTables installation!")
            raise InstallationError()

########################## USER INTERACTION #######################
def prompt_install(progname, install_prompt = None,
                   version=None, default="Y", *args, **kwargs):
    if version is not None:
        info = "%s %s" % (progname, version)
    else:
        info = "%s" % progname

    if install_prompt is None:
        install_prompt = "May I download and install %s?"
        
    query = install_prompt % info
    return prompt_yes_no(query, default=default)

def prompt_install_path(progname, default):
    query = "Where should %s be installed?" % progname
    path = prompt_user(query, default)
    return fix_path(path)

def prompt_yes_no(query, default="Y"):
    """Prompt user with query, given default and return boolean response

    Returns True if the user responds y/Y/Yes, and False if n/N/No
    """
    # Loop until we get a valid response
    while True:
        # Query user and get response
        print >>sys.stderr, "%s (Y/n) [%s] " % (query, default),
        sys.stdin.flush()
        response = raw_input().strip().lower()
        if len(response) == 0:
            response = default.strip().lower()
            
        if response.startswith("y"):
            return True
        elif response.startswith("n"):
            return False
        else:
            print >>sys.stderr, "Please enter yes or no."
        
def prompt_user(query, default=None, choices=None):
    """Prompt user with query, given default answer and optional choices
    """
    
    if choices is None:
        prompt = str(query)
    else:
        try:
            # Uniquify and convert to strings
            str_choices = list(str(choice) for choice in set(choices))
            assert(len(str_choices) > 1)
            lower_choices = list(choice.lower() for choice in str_choices)
            prompt = "%s (%s)" % (query, " / ".join(choices))
        except (AssertionError, TypeError):
            die("Invalid choice list: %s" % choices)
        
    # Loop until we get a valid response
    while True:
        # Query user and get response
        if default is None:
            msg = str(prompt)
        else:
            msg = "%s [%s] " % (prompt, default)

        print >>sys.stderr, msg,
        sys.stdin.flush()
        response = raw_input().strip()

        if len(response) == 0:  # User didn't enter a response
            if default is None:
                print >>sys.stderr, "Response required."
            else:
                return default
        elif choices is None:
            return response
        else:
            # Ensure the user picked from the set of choices
            matches = []
            for choice in lower_choices:
                if choice.startswith(response) or response.startswith(choice):
                    matches.append(choice)

            matched = len(matches)
            if matched == 0:
                print >>sys.stderr, "Invalid answer: %s" % response
                print >>sys.stderr, "Please select one of: (%s)" % \
                    ",".join(str_choices)
            elif matched == 1:
                return matches[0]
            else:
                print >>sys.stderr, ("Response matched multiple choices."
                                     " Please be more specific.")
                
def die(message):
    print >>sys.stderr, str(message)
    sys.exit(1)

####################### END COMMON CODE BODY #####################


############################## MAIN #########################
def main(args=sys.argv[1:]):
    # Set up shell details
    try:
        shell_name = os.path.basename(os.environ["SHELL"])
    except KeyError:
        shell_name = None
    shell = ShellManager(shell_name)
                
    try:
        # Set up arch_home
        arch_home = setup_arch_home()
        
        # Set up python home
        python_home, default_python_home = setup_python_home(arch_home)
        # Add python_home to PYTHONPATH
        prompt_add_to_env(shell, "PYTHONPATH", python_home)
        
        # Set up bin directory
        script_home, default_script_home = setup_script_home(arch_home)
        # Add script_home to PATH
        prompt_add_to_env(shell, "PATH", script_home)

        # Maybe create pydistutils.cfg
        prompt_create_cfg(arch_home, python_home, default_python_home,
                          script_home, default_script_home)
            
        # Add HDF5, if necessary
        hdf5_dir = prompt_install_hdf5(arch_home)
        if hdf5_dir:
            print >>sys.stderr, ("\nPyTables uses the environment variable"
                                 " HDF5DIR to locate HDF5.")
            prompt_set_env(shell, "HDF5_DIR", hdf5_dir)


        # Add Numpy, if necessary
        prompt_install_numpy()

        # Install segtools (and dependencies)
        prompt_install_segtools(arch_home)

        # Test package installations
        prompt_test_packages(arch_home=arch_home)

        print >>sys.stderr, "\n=========== Installation complete ============="
        
    finally:  # Clean up
        shell.close()
    
########################### GET VERSION ########################
def get_segtools_version():
    """Returns segtools version as a string or None if not found or installed
    
    Temporarily removes '.' from sys.path during installation to prevent
    finding segtools in current directory (but uninstalled)
    """
    dir = os.getcwd()
    index = None
    if dir in sys.path:
        index = sys.path.index(dir)
        del sys.path[index]
        
    try:
        try:
            import segtools
            return segtools.__version__
        except (AttributeError, ImportError):
            return None
    finally:
        if index is not None:
            sys.path.insert(index, dir)

##################### SPECIFIC PROGRAM INSTALLERS ################
def prompt_install_segtools(arch_home):
    return _installer("segtools", install_segtools, get_segtools_version,
                      install_prompt = EASY_INSTALL_PROMPT, arch_home=arch_home)
         
def install_segtools(arch_home, *args, **kwargs):
    segtools_dir = prompt_install_path("segtools", arch_home)
    install_dir = install_script("segtools", segtools_dir,
                                 SEGTOOLS_INSTALL_SCRIPT, url=SEGTOOLS_URL)
    return install_dir

            
if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
from __future__ import division, with_statement

__version__ = "$Revision: 90 $"

"""
html.py

HTML utilities for segtools
"""

import os
import re
import sys
import time

from .common import check_clobber, die, get_bed_metadata, make_divfilename, make_id, make_filename, make_tabfilename, NICE_EXTS, template_substitute

MNEMONIC_TEMPLATE_FILENAME = "mnemonic_div.tmpl"
HEADER_TEMPLATE_FILENAME = "html_header.tmpl"
FOOTER_TEMPLATE_FILENAME = "html_footer.tmpl"


def list2html(list, code=False, link=False):
    """
    If link is True: each element of the list should be a (divID, label) tuple
    """
    result = "<ul>"
    entrystr = "%s"
    if link:
        entrystr = '<a href="#%s">%s</a>'
    if code:
        entrystr = "<code>%s</code>"

    entrystr = "<li>%s</li>" % entrystr
    for entry in list:
        result += entrystr % entry
    return result + "</ul>"

def tab2html(tabfile, header=True):
    """
    Given a tab file table with a header row, generates an html string which
    for pretty-display of the table data.
    """
    result = '\n<table border="1" cellpadding="4" cellspacing="1">'
    # if the tabfile exists, write in htmlhandle an html table
    with open(tabfile) as ifp:
        if header:
            # first read the first line, and write the header row
            line = ifp.readline()
            fields = line.split("\t")
            result += "<tr>"
            for f in fields:
                result += '<td style="background-color: rgb(204, 204, 204)">%s</td>' % f
            result += "</tr>"
        
        for line in ifp:
            result += "<tr>"
            fields = line.split("\t")
            for f in fields: 
                result += "<td>%s</td>" % f
            result += "</tr>"
    result += "</table>\n"
    return result

def find_output_files(dirpath, namebase, d={}, tag=""):
    exts = NICE_EXTS
    # Add filenames of common present files to dict
    for extname, ext in exts.iteritems():
        filename = make_filename(dirpath, namebase, ext)
        if os.path.isfile(filename):
            key = "%s%sfilename" % (tag, extname)
            assert key not in d
            d[key] = filename

    return d

def form_template_dict(dirpath, namebase, module=None,
                       extra_namebases={}, **kwargs):
    """
    Given information about the current validation, generates a dictionary
    suitable for HTML template substitution.

    The output directory (dirpath) is searched for files of the form:
    <namebase>.<ext> for common exts. If found, the filename is linked
    in under the variable <ext>filename
    
    extra_namebases: a dict of tag -> namebase string
    For each extra namebase, any found files will be linked under
    <tag><ext>filename, as opposed to the main namebase, which is just
    under <ext>filename.

    module: The name of the module generating the dict. If specified,
    an id variable will be generated based upon the module and dirpath
    (a pseudo-unique identifier for the div file).

    Any keyword arguments of the form <prefix>tablenamebase
    will be assumed to refer to a table tab files and will be converted to
    html form and linked in under a variable named <prefix>table.
    The corresponding tab file is linked under a variable named
    <prefix>tablefilename.

    Any other keyword args supplied are linked into the dictionary.
    """
    # Find default files for all namebases
    d = {}
    find_output_files(dirpath, namebase, d=d)
    for tag, nb in extra_namebases.iteritems():
        find_output_files(dirpath, nb, d=d, tag=tag)

    if module is not None:
        arg = "id"
        assert arg not in d
        d[arg] = make_id(module, dirpath)
        
    for arg, val in kwargs.iteritems():
        index = arg.rfind("tablenamebase")
        if index >= 0: # arg ends with "tablenamebase"
            tablefilename = make_tabfilename(dirpath, val)
            if os.path.isfile(tablefilename):
                # Save html table under variable
                prefix = arg[0:index]
                arg = "%stable" % prefix
                val = tab2html(tablefilename)
                # Save table tab file under separate variable
                filearg = "%stablefilename" % prefix
                assert filearg not in d
                d[filearg] = tablefilename
            # else leave arg and val the way they were
        assert arg not in d  # Make sure wer're not overwriting anything
        d[arg] = val
            
    return d

def write_html_div(dirpath, namebase, html, clobber=False):
    """
    Write the given html div string to an appropriate file
    """
    filename = make_divfilename(dirpath, namebase)
    check_clobber(filename, clobber)
    
    with open(filename, "w") as ofp:
        print >>ofp, html

def save_html_div(template_filename, dirpath, namebase,
                  clobber=False, **kwargs):
    fields = form_template_dict(dirpath, namebase, **kwargs)
    
    try:
        html = template_substitute(template_filename)(fields)
    except KeyError, e:
        print >>sys.stderr, "Skipping HTML output. Missing data: %s" % e
        return
    
    write_html_div(dirpath, namebase, html, clobber=clobber)

def form_mnemonic_div(mnemonicfile):
    fields = {}
    fields["tabfilename"] = mnemonicfile
    fields["table"] = tab2html(mnemonicfile)
    div = template_substitute(MNEMONIC_TEMPLATE_FILENAME)(fields)
    return div
    
def form_html_header(bedfilename, modules):
    segtool, segtracks = get_bed_metadata(bedfilename)
    fields = {}
    fields["bedfilename"] = os.path.basename(bedfilename)
    fields["numsegtracks"] = len(segtracks)
    fields["segtracks"] = list2html(segtracks, code=True)
    fields["segtool"] = segtool
    fields["bedmtime"] = time.strftime(
        "%m/%d/%Y %I:%M:%S %p", time.localtime(os.path.getmtime(bedfilename)))
    fields["modules"] = list2html(modules, link=True) 

    header = template_substitute(HEADER_TEMPLATE_FILENAME)(fields)
    return header

def form_html_footer():
    return template_substitute(FOOTER_TEMPLATE_FILENAME)()

def find_divs(rootdir=os.getcwd()):
    """Look one level deep in directory, adding any .div files found.
    """
    divs = []
    for foldername in sorted(os.listdir(rootdir)):
        folderpath = os.path.join(rootdir, foldername)
        if os.path.isdir(folderpath):
            for filename in sorted(os.listdir(folderpath)):
                root, ext = os.path.splitext(filename)
                if ext == ".div":
                    filepath = os.path.join(folderpath, filename)
                    divs.append(filepath)
                    print >>sys.stderr, "Found div: %s" % filename
    return divs

def parse_args(args):
    from optparse import OptionParser
    usage = "%prog [OPTIONS] BEDFILE"
    version = "%%prog %s" % __version__
    parser = OptionParser(usage=usage, version=version)


    parser.add_option("--clobber", action="store_true",
                      dest="clobber", default=False,
                      help="Overwrite existing output files if the specified"
                      " directory already exists.")
    parser.add_option("--mnemonic-file", dest="mnemonicfile",
                      default=None,
                      help="If specified, this mnemonic mapping will be"
                      " included in the report (this should be the same"
                      " mnemonic file used by the individual modules)")
    parser.add_option("--results-dir", dest="resultsdir", default=".",
                      help="This should be the directory containing all the"
                      " module output directories (`ls` should return things"
                      " like \"length_distribution/\", etc)"
                      " [default: %default]")
    parser.add_option("-o", "--outfile", 
                      dest="outfile", default="index.html",
                      help="HTML report file (must be in current directory"
                      " [default: %default]")

    options, args = parser.parse_args(args)

    if options.outfile != os.path.basename(options.outfile):
        parser.error("Output file must be in current directory (otherwise the reource paths get all messed up)")
    
    if len(args) < 1:
        parser.error("Insufficient number of arguments")

    return options, args
    
def main(args=sys.argv[1:]):
    options, args = parse_args(args)
    bedfilename = args[0]
    mnemonicfile = options.mnemonicfile
    outfile = options.outfile
    check_clobber(outfile, options.clobber)
    
    divs = find_divs(options.resultsdir)
    if len(divs) == 0:
        die("Unable to find any module .div files. Make sure to run this from the parent directory of the module output directories or specify the --results-dir option")

    body = []
    modules = []
    if mnemonicfile is not None:
        body.append(form_mnemonic_div(mnemonicfile))

    regex = re.compile('"module" id="(.*?)".*?<h.>.*?</a>\s*(.*?)\s*</h.>',
                       re.DOTALL)
    for div in divs:
        with open(div) as ifp:
            divstring = "".join(ifp.readlines())
            matching = regex.search(divstring)
            assert matching
            module = (matching.group(1), matching.group(2))
            modules.append(module)
            body.append(divstring)
            
    header = form_html_header(bedfilename, modules)
    footer = form_html_footer()

    components = [header] + body + [footer]
    separator = "<br /><hr>"
    with open(outfile, "w") as ofp:
        print >>ofp, separator.join(components)
        
if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
from __future__ import division, with_statement

__version__ = "$Revision: 90 $"

"""
label_transition.py:

Provides command-line and package entry points for analyzing the observed
segmentation label transitions in the given BED-formatted segmentation.


Accepts an input file containing a matrix of transition
probabilities and generates a heatmap of the matrix
in pdf and png formats.

Optionally generates a heatmap of the ratio of reciprocal
transition probabilities (a->b/b->a)

Optionally generates a graphviz image of a thresholded
form of the transition matrix

XXX: Graphiz currently assumes integer labels in sorted order in file

"""

# A package-unique, descriptive module name used for filenames, etc
# Must be the same as the folder containing this script
MODULE = "label_transition"


import os
import sys


from contextlib import contextmanager
from functools import partial
from numpy import array, empty, sort, where, zeros
from rpy2.robjects import r, rinterface, numpy2ri

# XXX: Do this without the kludgy constants
from .common import check_clobber, die, get_ordered_labels, image_saver, load_segmentation, load_genome, make_dotfilename, make_filename, make_pdffilename, make_pngfilename, make_namebase_summary, make_tabfilename, map_mnemonics, OutputMasker, r_source, SEGMENT_START_COL, SEGMENT_END_COL, SEGMENT_LABEL_KEY_COL, setup_directory, tab_saver

from .html import save_html_div


NAMEBASE = "%s" % MODULE
NAMEBASE_SUMMARY = make_namebase_summary(NAMEBASE)
NAMEBASE_GRAPH = os.extsep.join([NAMEBASE, "graph"])
NAMEBASE_PARAMS = os.extsep.join([NAMEBASE, "params"])

HTML_TEMPLATE_FILENAME = "transition_div.tmpl"
HTML_TITLE = "Segment label transitions"

P_THRESH = 0.15  # Default
Q_THRESH = 0.0

BASE_WEIGHT = 0.3  # Base edge/arrow weight
EDGE_WEIGHT = 2.7
ARROW_WEIGHT = 1.7

PNG_SIZE = 600  # px
PNG_SIZE_PARAMS = 800  # px

def start_R():
    r_source("common.R")
    r_source("transition.R")

## Returns array of row-normalized probabilities corresponding
## to transition probabilities between labels
## Start labels are rows, end labels are cols
def calc_transitions(segmentation):
    labels = segmentation.labels
    self_transition_count = 0
    # NxN matrix for transition counts
    counts = zeros((len(labels), len(labels)), dtype="int")
    for segments in segmentation.chromosomes.itervalues():
        # Inch along rows, looking at all pairs
	row_it = iter(segments)
        row1 = None
        for row2 in row_it:
            if row1 is not None:  # Skip first loop
                # Increment transition only if directly adjacent
                if row1[SEGMENT_END_COL] == row2[SEGMENT_START_COL]:
                    label_key1 = row1[SEGMENT_LABEL_KEY_COL]
                    label_key2 = row2[SEGMENT_LABEL_KEY_COL]
                    counts[label_key1, label_key2] += 1
                    if label_key1 == label_key2:
                        if self_transition_count == 0:
                            print >>sys.stderr, "WARNING: unexpected self-transition %s -> %s\n\tRow 1: %s\n\tRow 2: %s" % (labels[label_key1], labels[label_key2], row1, row2)
                        self_transition_count += 1

            row1 = row2  # Inch forward
            
    if self_transition_count > 0:
        print >>sys.stderr, "WARNING: %d self-transitions observed" % self_transition_count
        
    # Row-normalize to turn counts into probabilities
    probs = empty(counts.shape, dtype='double')
    for i, row in enumerate(counts):
        probs[i] = row / sum(row)

    return labels, probs


def save_tab(labels, probs, dirpath, clobber=False):
    ordered_keys, labels = get_ordered_labels(labels)
    
    # Get fieldnames in order
    fieldnames = list(labels[key] for key in ordered_keys)
    with tab_saver(dirpath, NAMEBASE, fieldnames,
                   clobber=clobber, header=False) as saver:
        for start_key in ordered_keys:
            prob_row = probs[start_key]
            row = {}
            for end_key in ordered_keys:
                row[labels[end_key]] = "%.5f" % prob_row[end_key]
            saver.writerow(row)

def save_summary_tab(labels, probs, dirpath, mnemonics=None, clobber=False):
    ordered_keys, labels = get_ordered_labels(labels, mnemonics)
    # Get fieldnames in order
    assert "label" not in labels.keys()
    fieldnames = ["label"] + list(labels[key] for key in ordered_keys)
    with tab_saver(dirpath, NAMEBASE_SUMMARY, fieldnames,
                   clobber=clobber) as saver:
        for start_key in ordered_keys:
            prob_row = probs[start_key]
            row = {}
            row["label"] = labels[start_key]
            for end_key in ordered_keys:
                row[labels[end_key]] = "%.5f" % prob_row[end_key]
            saver.writerow(row)

def save_html(dirpath, p_thresh, q_thresh, clobber=False):
    extra_namebases = {"graph": NAMEBASE_GRAPH}
    
    if p_thresh > 0:
        thresh = "P > %s" % p_thresh
    elif q_thresh > 0:
        thresh = "P above %.2th quantile" % q_thresh

    save_html_div(HTML_TEMPLATE_FILENAME, dirpath, NAMEBASE, clobber=clobber,
                  module=MODULE, extra_namebases=extra_namebases,
                  title=HTML_TITLE, thresh=thresh)

def save_plot(dirpath, ddgram=False, clobber=False, mnemonics=[],
              verbose=False):
    start_R()

    tabfilename = make_tabfilename(dirpath, NAMEBASE)
    if not os.path.isfile(tabfilename):
        die("Unable to find tab file: %s" % tabfilename)
        
    with image_saver(dirpath, NAMEBASE,
                     height=PNG_SIZE,
                     width=PNG_SIZE,
                     clobber=clobber,
                     verbose=verbose):
        r.plot(r["plot.transition"](tabfilename,
                                    mnemonics=mnemonics,
                                    ddgram=ddgram))

def plot_gmtk_transitions(filename, dirpath, clobber=False, verbose=False,
                          mnemonics=[]):
    """Plots transition probabilities from gmtk params file.
    """
    assert filename is not None and os.path.isfile(filename)
    
    start_R()
    
    # Plot transition probabilies
    with image_saver(dirpath, NAMEBASE,
                     height=PNG_SIZE,
                     width=PNG_SIZE,
                     clobber=clobber,
                     verbose=verbose):
        r.plot(r["plot.gmtk.transition"](filename,
                                         mnemonics=mnemonics))

    # Plot track parameters
    with image_saver(dirpath, NAMEBASE_PARAMS,
                     height=PNG_SIZE_PARAMS,
                     width=PNG_SIZE_PARAMS,
                     clobber=clobber,
                     verbose=verbose):
        r.plot(r["plot.gmtk.params"](filename,
                                     mnemonics=mnemonics))

def save_graph(labels, probs, dirpath, q_thresh=Q_THRESH, p_thresh=P_THRESH,
               clobber=False, mnemonics=[]):
    assert labels is not None and probs is not None
    try:
        import pygraphviz as pgv
    except ImportError:
        print >>sys.stderr, "Unable to load PyGraphviz library. Skipping transition graph"
        return
    
    dotfilename = make_dotfilename(dirpath, NAMEBASE_GRAPH)
    check_clobber(dotfilename, clobber)
    pngfilename = make_pngfilename(dirpath, NAMEBASE_GRAPH)
    check_clobber(pngfilename, clobber)
    pdffilename = make_pdffilename(dirpath, NAMEBASE_GRAPH)
    check_clobber(pdffilename, clobber)

    # Replace labels with mnemonic labels, if mnemonics are given
    ordered_keys, labels = get_ordered_labels(labels, mnemonics)
    
    # Threshold in order dependent upon flag
    if q_thresh > 0:
        quantile = r['matrix.find_quantile'](probs, q_thresh)[0]
        print >>sys.stderr, "Removing edges below %.4f" % float(quantile)
        probs[probs < quantile] = 0
    elif p_thresh > 0:
        print >>sys.stderr, "Removing edges below %.4f" % p_thresh
        probs[probs < p_thresh] = 0
        
    # Create graph out of non-zero edges
    G = pgv.AGraph(strict=False, directed=True)

    rows, cols = where(probs > 0)
        
    max_val = probs.max()
    min_val = probs[probs > 0].min()
    weights = (probs - min_val)/(max_val-min_val) + BASE_WEIGHT
    weights[weights < BASE_WEIGHT] = 0
    for row, col in zip(rows, cols):
        weight = weights[row, col]
        G.add_edge(labels[row], labels[col],
                   penwidth=str(EDGE_WEIGHT * weight),
                   arrowsize=str(ARROW_WEIGHT * weight))

    G.write(dotfilename)

    print >>sys.stderr, "Drawing graph...",
    # Mask obnoxious font errors
    sys.stderr = OutputMasker(sys.stderr)
    G.layout()
    sys.stderr = sys.stderr.restore()
    try:
        G.draw(pngfilename)
    except:
        print >>sys.stderr, "Failed to draw png graph"
        
    try:
        G.draw(pdffilename)
    except:
        print >>sys.stderr, "Failed to draw pdf graph"
        
    print >>sys.stderr, "done"

def create_mnemonic_file(gmtk_file, dirpath, clobber=False, namebase = None):
    """Generate a mnemonic file for GMTK params with R clustering code.

    Calls R code that writes mnemonics to a file.
    Returns name of created mnemonic file
    """
    start_R()

    if namebase is None:
        gmtk_filebase = os.path.basename(gmtk_file)
        if gmtk_filebase.endswith(".params"):
            namebase = gmtk_filebase[:-7]
        else:
            namebase = gmtk_filebase

    mnemonic_base = os.extsep.join([namebase, "mnemonics"])
    
    filename = make_tabfilename(dirpath, mnemonic_base)
    check_clobber(filename, clobber)

    # Create mnemonic file via R
    try:
        r_filename = r["make.gmtk.mnemonic.file"](gmtk_file, filename=filename)
        mnemonicfilename = str(r_filename[0])

        print >>sys.stderr, "Generated mnemonic file: %s" % mnemonicfilename
        return mnemonicfilename
    except rinterface.RRuntimeError:
        print >>sys.stderr, "ERROR: Failed to create mnemonic file. Continuing without mnemonics."
        return None
    
def get_num_gmtk_labels(gmtk_file):
    """Load number of labels from gmtk params file."""
    start_R()
    
    num_labels = int(r["num.gmtk.labels"](gmtk_file)[0])
    return num_labels
    
def get_default_labels(num_labels):
    """Generate default labels (0 through num_labels-1)"""
    return dict([(val, str(val)) for val in range(0, num_labels)])

def load_gmtk_transitions(gmtk_file):
    """Loads probabilites from a gmtk_file, through R.

    Returns probs as a numpy.array
    """
    start_R()

    r_data = r["read.gmtk.transition"](gmtk_file)
    # Rpy automatically transposes, so need to transpose it back
    probs = array(r_data, dtype="double").transpose()
    return probs
    

## Package entry point
def validate_gmtk(gmtk_file, dirpath, ddgram=False, p_thresh=P_THRESH,
                  q_thresh=Q_THRESH, noplot=False, nograph=False,
                  clobber=False, mnemonicfilename=None):
    setup_directory(dirpath)

    if not os.path.isfile(gmtk_file):
        die("Could not find GMTK file: %s" % gmtk_file)

    # If mnemonics weren't specified, let's create a mnemonic file!
    if mnemonicfilename is None:
        mnemonicfilename = create_mnemonic_file(gmtk_file, dirpath,
                                                clobber=clobber)

    num_labels = get_num_gmtk_labels(gmtk_file)
    labels = get_default_labels(num_labels)
    mnemonics = map_mnemonics(labels, mnemonicfilename)

    if not noplot:
        plot_gmtk_transitions(gmtk_file, dirpath, mnemonics=mnemonics,
                              clobber=clobber)
    
    if not nograph:
        probs = load_gmtk_transitions(gmtk_file)
        save_graph(labels, probs, dirpath, clobber=clobber,
                   p_thresh=p_thresh, q_thresh=q_thresh, mnemonics=mnemonics)

    save_html(dirpath, p_thresh=p_thresh, q_thresh=q_thresh,
              clobber=clobber)
    
## Package entry point
def validate(bedfilename, dirpath, ddgram=False, p_thresh=P_THRESH,
             q_thresh=Q_THRESH, noplot=False, nograph=False,
             clobber=False, mnemonicfilename=None):
    setup_directory(dirpath)

    segmentation = load_segmentation(bedfilename)
    assert segmentation is not None
    
    # Calculate transition probabilities for each label
    labels, probs = calc_transitions(segmentation)
        
    mnemonics = map_mnemonics(labels, mnemonicfilename)
    save_summary_tab(labels, probs, dirpath,
                     clobber=clobber, mnemonics=mnemonics)
    save_tab(labels, probs, dirpath, clobber=clobber)
    if not noplot:
        save_plot(dirpath, ddgram=ddgram,
                  clobber=clobber, mnemonics=mnemonics)
            
    if not nograph:
        save_graph(labels, probs, dirpath, clobber=clobber,
                   p_thresh=p_thresh, q_thresh=q_thresh, mnemonics=mnemonics)

    save_html(dirpath, p_thresh=p_thresh, q_thresh=q_thresh,
              clobber=clobber)

def parse_options(args):
    from optparse import OptionParser, OptionGroup

    usage = "%prog [OPTIONS] BEDFILE"
    version = "%%prog %s" % __version__
    parser = OptionParser(usage=usage, version=version)
    
    parser.add_option("--clobber", action="store_true",
                      dest="clobber", default=False,
                      help="Overwrite existing output files if the specified"
                      " directory already exists.")
    parser.add_option("--noplot", action="store_true", 
                      dest="noplot", default=False,
                      help="Do not generate transition plots")
    parser.add_option("--nograph", action="store_true", 
                      dest="nograph", default=False,
                      help="Do not generate transition graph")
    parser.add_option("--mnemonic-file", dest="mnemonic_file",
                      default=None,
                      help="If specified, labels will be shown using"
                      " mnemonics found in this file")
    parser.add_option("-o", "--outdir", 
                      dest="outdir", default="%s" % MODULE,
                      help="File output directory (will be created"
                      " if it does not exist) [default: %default]")
    
    group = OptionGroup(parser, "Transition frequency plot options")
    group.add_option("--dd", "--dendrogram", dest="ddgram",
                     default=False, action="store_true",
                     help="include dendrogram along edge of levelplot"
                     " [default: %default]")
    parser.add_option_group(group)

    group = OptionGroup(parser, "Transition graph options")
    group.add_option("-p", "--prob-threshold", dest="p_thresh",
                     type="float", default=P_THRESH,
                     help="ignore all transitions with probabilities below"
                     " this absolute threshold [default: %default]")
    group.add_option("-q", "--quantile-threshold", dest="q_thresh",
                     type="float", default=Q_THRESH,
                     help="ignore transitions with probabilities below this"
                     " probability quantile [default: %default]")
    parser.add_option_group(group)

    group = OptionGroup(parser, "Non-segmentation files")
    parser.add_option("--gmtk-params", dest="gmtk_file",
                      default=None,
                      help="If specified, parameters in the given GMTK"
                      " file will be used to generate plots instead of the"
                      " observed transitions in the BEDFILE. The BEDFILE"
                      " will not be used")
    parser.add_option_group(group)

    (options, args) = parser.parse_args(args)

    if len(args) < 1 and options.gmtk_file is None:
        parser.error("Insufficient number of arguments")

    if options.p_thresh > 0 and options.q_thresh > 0:
        parser.error("Cannot specify both absolute and quantile thresholds")
    if options.q_thresh < 0 or options.q_thresh > 1:
        parser.error("Quantile threshold should be in range [0, 1]")
    if options.p_thresh < 0 or options.p_thresh > 1:
        parser.error("Probability threshold should be in range [0, 1]")
    
    return (options, args)
    
## Command-line entry point
def main(args=sys.argv[1:]):
    (options, args) = parse_options(args)

    if options.gmtk_file is not None:
        validate_gmtk(options.gmtk_file, options.outdir,
                      p_thresh=options.p_thresh, q_thresh=options.q_thresh,
                      clobber=options.clobber, noplot=options.noplot,
                      nograph=options.nograph,
                      mnemonicfilename=options.mnemonic_file)
        
    else:
        bedfilename=args[0]
        validate(bedfilename, options.outdir, ddgram=options.ddgram,
                 p_thresh=options.p_thresh, q_thresh=options.q_thresh,
                 clobber=options.clobber, noplot=options.noplot,
                 nograph=options.nograph,
                 mnemonicfilename=options.mnemonic_file)
        
if __name__ == "__main__":
    sys.exit(main())



from __future__ import division, with_statement

__version__ = "$Revision: 90 $"

"""
common.py:

Assorted utility functions and classes common or useful to most of segtools.


Author: Orion Buske <orion.buske@gmail.com>
"""

import os
import re
import sys

from collections import defaultdict
from contextlib import closing, contextmanager
from functools import partial
from genomedata import Genome
from gzip import open as _gzip_open
from numpy import array, empty, iinfo
from pkg_resources import resource_filename, resource_string
from string import Template
from tabdelim import DictReader, DictWriter
from rpy2.robjects import r, rinterface, numpy2ri
# numpy2ri imported for side-effect

try:
    PKG = __package__  # Python 2.6
except NameError:
    PKG = "segtools"

PKG_R = os.extsep.join([PKG, "R"])
PKG_RESOURCE = os.extsep.join([PKG, "resources"])
    
from .bed import read_native

EXT_GZ = "gz"
EXT_PDF = "pdf"
EXT_PNG = "png"
EXT_TAB = "tab"
EXT_DOT = "dot"
EXT_HTML = "html"
EXT_DIV = "div"
EXT_SLIDE = os.extsep.join(["slide", EXT_PNG])
EXT_THUMB = os.extsep.join(["thumb", EXT_PNG])  # for thumbnails
EXT_SUMMARY = os.extsep.join(["summary", EXT_TAB])

IMG_EXTS = [EXT_PNG, EXT_PDF, EXT_SLIDE, EXT_THUMB]
NICE_EXTS = dict(tab=EXT_TAB, pdf=EXT_PDF, png=EXT_PNG, html=EXT_HTML,
                 div=EXT_DIV, slide=EXT_SLIDE, thumb=EXT_THUMB,
                 summary=EXT_SUMMARY, dot=EXT_DOT)

SUFFIX_GZ = os.extsep + EXT_GZ

LABEL_ALL = "all"

THUMB_SIZE = 100

SEGMENT_START_COL = 0
SEGMENT_END_COL = 1
SEGMENT_LABEL_KEY_COL = 2

#FEATURE_FIELDS = ["chrom", "source", "name", "start", "end", "score", "strand"]

r_filename = partial(resource_filename, PKG_R)
template_string = partial(resource_string, PKG_RESOURCE)


## Wrapper for segmentation data object
class Segmentation(object):
    """
    chromosomes: a dict
      key: chromosome name
      val: segments: numpy.ndarray(each row = chromStart, chromEnd, label_key)
           sorted by chromStart
    labels: a dict
      key: int ("label_key")  (a unique id)
      val: printable (str or int)  (what's in the actual BED file)
      
    tracks: a list of track names that were used to obtain the segmentation
    segtool: the tool that was used to obtain this segmentation (e.g. segway)  
    """

    def __init__(self, chromosomes, labels, tracks, segtool):
        self.chromosomes = chromosomes
        self.labels = labels
        self.tracks = tracks
        self.segtool = segtool


class OutputMasker:
    """Class to mask the output of a stream.

    Suggested usage:
      sys.stout = OutputMasker(sys.stdout)  # Start masking
      <commands with stout masked>  # Masked commands
      sys.stdout = sys.stdout.restore()  # Stop masking
    """
    def __init__(self, stream=None):
        self._stream = stream
    def write(self, string):
        pass  # Mask output
    def writelines(self, lines):
        pass  # Mask output
    def restore(self):
        return self._stream
        
## Wrapper for gff/gtf feature data
# class Feature(object):
#     def __init__(self, line=None, tokens=None):
#         assert (line or tokens) and not (line and tokens)
#         if tokens is None:
#             tokens = line.strip().split("\t")
            
#         self.__dict__ = dict(zip(FEATURE_FIELDS, tokens))
            
#         # Make zero-based, exclusive end:
#         #   http://genome.ucsc.edu/FAQ/FAQformat#format3
#         self.start = int(self.start) - 1
#         self.end = int(self.end)
        
        
## UTILITY FUNCTIONS
        
## Die with error message
def die(msg="Unexpected error"):
    print >> sys.stderr, "\nERROR: %s\n" % msg
    sys.exit(1)
        
def make_filename(dirpath, basename, ext):
    return os.path.join(dirpath, os.extsep.join([basename, ext]))

make_tabfilename = partial(make_filename, ext=EXT_TAB)
make_htmlfilename = partial(make_filename, ext=EXT_HTML)
make_divfilename = partial(make_filename, ext=EXT_DIV)
make_pngfilename = partial(make_filename, ext=EXT_PNG)
make_pdffilename = partial(make_filename, ext=EXT_PDF)
make_thumbfilename = partial(make_filename, ext=EXT_THUMB)
make_slidefilename = partial(make_filename, ext=EXT_SLIDE)
make_summaryfilename = partial(make_filename, ext=EXT_SUMMARY)
make_dotfilename = partial(make_filename, ext=EXT_DOT)

def make_namebase_summary(namebase):
    return os.extsep.join([namebase, "summary"])

def make_id(modulename, dirpath):
    return "_".join([modulename, os.path.basename(dirpath)])

def check_clobber(filename, clobber):
    if (not clobber) and os.path.isfile(filename):
        die("Output file: %s already exists! Use --clobber to overwrite!" % filename)
        
def gzip_open(*args, **kwargs):
    return closing(_gzip_open(*args, **kwargs))

def maybe_gzip_open(filename, *args, **kwargs):
    if filename.endswith(SUFFIX_GZ):
        return gzip_open(filename, *args, **kwargs)
    else:
        return open(filename, *args, **kwargs)
    
def fill_array(scalar, shape, dtype=None, *args, **kwargs):
    if dtype is None:
        dtype = array(scalar).dtype

    res = empty(shape, dtype, *args, **kwargs)
    res.fill(scalar)
    return res


## XXX: No known usage
@contextmanager
def none_contextmanager():
    yield None

def r_source(filename):
    """
    Simplify importing R source in the package
    """
    try:
        r.source(r_filename(filename))
    except rinterface.RRuntimeError:
        die("Failed to load R package: %s\n" % filename)

def template_substitute(filename):
    """
    Simplify import resource strings in the package
    """
    return Template(template_string(filename)).substitute
        
@contextmanager
def tab_saver(dirpath, basename, fieldnames, comment=None,
              clobber=False, header=True, verbose=True):
    """
    Saves data to tab file (yeilds DictWriter)
    """
    if verbose:
        print >>sys.stderr, "Saving tab file...",
        
    outfilename = make_tabfilename(dirpath, basename)
    check_clobber(outfilename, clobber)
    with open(outfilename, "w") as outfile:
        if comment is not None and len(comment) > 0:
            if comment != str(comment):  # Not a single string
                for line in comment:
                    print >> outfile, "# %s" % line
            else:
                print >> outfile, "# %s" % comment
        yield DictWriter(outfile, fieldnames,
                         extrasaction="ignore", header=header)
        
    if verbose:
        print >>sys.stderr, "done"

## OLD IMAGE SAVER
## Generator to save an R plot to both png and pdf with only one plot
## Yields to caller to plot, then saves images
# @contextmanager
# def image_saver(dirpath, basename, clobber=False, **kwargs):
#     png_filename = make_filename(dirpath, basename, EXT_PNG)
#     png_slide_filename = make_filename(dirpath,
#                                        "%s%s" % (basename, ".slide"),
#                                        EXT_PNG)
#     pdf_filename = make_filename(dirpath, basename, EXT_PDF)

#     # Make sure there is no silent overwrite
#     filenames = [png_filename, png_slide_filename, pdf_filename]
#     if not clobber:
#         file_exists = list(os.path.isfile(file) for file in filenames)
#         if any(file_exists):
#             die("Output file: %s already exists! Use --clobber to overwrite!" % filenames[file_exists.index(True)])
        
#     # Set up primary device
#     try:
#         r.png(png_filename, **kwargs)
#     except rinterface.RRuntimeError:
#         die('If unable to start PNG device, try `export DISPLAY=":1"` from the shell before re-running validation')
        
#     png_device = r['dev.cur']()
        
#     r['dev.control']("enable")  # Allow copying from device

#     print >>sys.stderr, "\tCreating images..."
#     yield  # Wait for plot

#     try:
#         # Make pdf copy
#         # Dingbats off because otherwise circles displayed as q's
#         r['dev.copy2pdf'](file=pdf_filename, useDingbats=False)
        
#         # Make slide copy
#         r['dev.print.slide'](png_slide_filename)
        
#         # Flush buffer and close plot
#         r['dev.off'](png_device)
        
#         print >>sys.stderr, "\tImages saved!"
#     except:
#         r['dev.off'](png_device)
#         print >> sys.stderr, "ERROR: R crashed. Some images might not be saved."

## CURRENT VERSION OF IMAGE_SAVER
@contextmanager
def image_saver(dirpath, basename, clobber = False, verbose = True,
                downsample = False, **kwargs):
    """
    Generator to save an R plot to both png and pdf with only one plot
    Yields to caller to plot, then saves images
    """
    # Make sure there is no silent overwrite
    filenames = list(make_filename(dirpath, basename, ext)
                     for ext in IMG_EXTS)
    for filename in filenames:
        check_clobber(filename, clobber)

    if verbose:
        print >>sys.stderr, "Creating images...",
        
    try:
        png_filename = make_pngfilename(dirpath, basename)
        r.png(png_filename, **kwargs)
        png_device = r["dev.cur"]()
        r["dev.control"]("enable")
    except rinterface.RRuntimeError:
        die('Image creation failed.\nIf unable to start PNG device, try setting (export/setenv) variable DISPLAY to ":1" from the (bash/c) shell before re-running validation.')
        
    yield  # Wait for plot
    
    # Use R routine to create all the other images (pdf, slide, etc)
    try:
        r["dev.print.images"](basename = basename, dirname = dirpath,
                              downsample = downsample, **kwargs)
        r["dev.off"](png_device)
    except rinterface.RRuntimeError:
        print >> sys.stderr, 'ERROR: Images might not have been saved!'

    if verbose:
        print >>sys.stderr, "done"

def get_bed_metadata(bedfilename):
    with maybe_gzip_open(bedfilename) as ifp:
        return parse_bed_header(ifp.readline())

## Gets track/tool info from BED file header if it exists
def parse_bed_header(line):
    regexp = re.compile('description="(.*) segmentation of (.*)"')

    segtool = "Missing from BED file"
    tracks = ["Missing from BED file"]

    matching = regexp.search(line)
    if matching:
        segtool = matching.group(1)
        tracks = matching.group(2).split(', ')
    return (segtool, tracks)


## Maps label_keys over segments
def map_segments(segments, labels, chrom_size):
    """
    converts a segment mapping into label_keys at every nucleotide position 
        in the chromosome
        
    e.g.  segment labeled 0 at position [0,4) followed by 
          segment labeled 1 at position [4,7) gets converted into:
          0000111
    """  
    segments_dtype = segments.dtype  # MA: the data type of segments
    segments_dtype_max = iinfo(segments_dtype).max # sentinel value		
    # MA: the maximum value supported by the type
    assert segments_dtype_max not in labels.keys()

    res = fill_array(segments_dtype_max, (chrom_size,), segments_dtype)

    # will overwrite in overlapping case
    for segment in segments:
        res[segment[SEGMENT_START_COL]:
                segment[SEGMENT_END_COL]] = segment[SEGMENT_LABEL_KEY_COL]

    return res

## Yields segment and the continuous corresponding to it, for each segment
##   in the chromosome inside of a supercontig
def loop_segments_continuous(chromosome, segmentation, verbose=True):
    try:
        segments = segmentation.chromosomes[chromosome.name]
    except KeyError:
        if verbose:
            print >>sys.stderr, "\t\tskipping: no data"
        raise StopIteration
    
    supercontig_iter = chromosome.itercontinuous()
    supercontig = None
    supercontig_last_start = 0
    segment_i = 0
    num_segments = len(segments)
    for segment in segments:
        segment_i += 1
        seg_start = segment[SEGMENT_START_COL]
        seg_end = segment[SEGMENT_END_COL]  # Exclusive

        while supercontig is None or seg_start >= supercontig.end:
            # Raise StopIteration if out of supercontigs
            supercontig, continuous = supercontig_iter.next()
            # Enforce increasing supercontig indices
            assert supercontig.start >= supercontig_last_start
            supercontig_last_start = supercontig.start

            if verbose:
                print >>sys.stderr, "\n\t\t%s) %d : %d" % (supercontig.name,
                                             supercontig.start,
                                             supercontig.end)

        if verbose and segment_i % 100 == 0:
            print >>sys.stderr, "\r\t\t\tSegment %d / %d" % (segment_i, num_segments),
            sys.stdout.flush()
            
        if seg_end <= supercontig.start:
            continue  # Get next segment

        yield segment, continuous[seg_start:seg_end]

    if verbose:
        print >>sys.stderr, "\r\t\t\tSegment %d / %d" % (segment_i, num_segments)
        

## Returns splice of continuous or sequece from supercontig within segment
## range. Datatype should be "continuous" or "sequence".
## Assumes supercontigs are non-overlapping and sorted by start ascending
## Returns empty array if no supercontig touches the given range
def get_supercontig_splice(chromosome, start, end, datatype="continuous"):
    if datatype not in ("continuous", "sequence"):
        die("Unknown data type in 'get_supercontig_splice!")
        
    # iterate through each supercontig
    saved = array([])
    for supercontig, continuous in chromosome.itercontinuous():
        #print >>sys.stderr, "Examining supercontig [%d, %d]" % (supercontig.start, supercontig.end)

        # Select appropriate data stream
        if datatype == "continuous":
            data = continuous
        else:
            data = supercontig.seq
            
        # XXX Make sure there are no OBOBS in indexing!
        if start >= supercontig.start and end <= supercontig.end:
            # Found supercontig that completely covers range
            return data[start - supercontig.start : end - supercontig.start]
        elif start >= supercontig.start and start < supercontig.end:
            # Overlapping supercontig end
            saved = array(data[start - supercontig.start : len(data) - 1])
        elif end > supercontig.start and end < supercontig.end:
            # Overlapping supercontig start
            data_splice = data[0 : end - supercontig.start]
            # Because supercontigs in sorted order, we can return here
            if saved.shape[0] > 0:
                return concatenate((saved, data_splice))
            else:
                return data_splice
        elif end < supercontig.start:
            # We've passed the appropriate range
            break

    return saved

## Ensures output directory exists and has appropriate permissions
def setup_directory(dirname):
    if not os.path.isdir(dirname):
        try:
            os.mkdir(dirname)
        except IOError:
            die("Error: Could not create output directory: %s" % (dirname))
    else:
        # Require write and execute permissions on existing dir
        if not os.access(dirname, os.W_OK | os.X_OK):
            die("Error: Output directory does not have appropriate permissions!")

## Given labels and mnemonics, returns an ordered list of label_keys
##   and a new labels dict mapping label_keys to label strings
## If no mnemonics are specified, returns the passed labels and
##   a label_key ordering
def get_ordered_labels(labels, mnemonics=[]):
    if mnemonics is not None and len(mnemonics) > 0:
        # Create key lookup dictionary
        key_lookup = {}  # old_label -> label_key
        for label_key, label in labels.iteritems():
            assert(label not in key_lookup)  # Enforce 1-to-1
            key_lookup[label] = label_key

        labels = {}
        ordered_keys = []
        for old_label, new_label in mnemonics:
            label_key = key_lookup[old_label]
            ordered_keys.append(label_key)
            labels[label_key] = new_label
    else:
        # Don't change labels, but specify ordering
        partial_int_labels = {}
        for key, label in labels.iteritems():
            try:
                partial_int_labels[int(label)] = key
            except ValueError:
                partial_int_labels[label] = key
        ordered_keys = list(partial_int_labels[key]
                            for key in sorted(partial_int_labels.keys()))
        
    return ordered_keys, labels
    
## Maps the provided labels to mnemonics (or some other mnemonic file
##   field specified ready to be fed into R.
## Returns a numpy.array of strings with a row of [oldlabel, newlabel] for
## every old_label, and their ordering specifies the desired display order
## Labels should be a dict of label_keys -> label strings
def map_mnemonics(labels, mnemonicfilename, field="mnemonic"):
    if mnemonicfilename is None:
        return []
    
    label_order, label_data = load_mnemonics(mnemonicfilename)
    str_labels = labels.values()

    mnemonics = []
    # Add mapping for labels in mnemonic file
    for old_label in label_order:
        new_label = label_data[old_label][field]
        if old_label in str_labels:
            mnemonics.append([old_label, new_label])

    # Add mapping for labels not in mnemonic file
    # Use ordering of label mapping
    for label_key in sorted(labels.keys()):
        label = labels[label_key]  # Get actual label (string)
        if label not in label_order:  # Then map to same name
            mnemonics.append([label, label])

    return array(mnemonics)
            
## Loads segmentation label descriptions and mnemonics
##   from a tab-delimited file with a header row
def load_mnemonics(filename):
    """
    Input file should have a tab-delimited row for each label, of the form:
               index    description    mnemonic
      e.g.     4    initiation (strong)    IS
    Returns a tuple of (label_order, label_data)
    
    label_order: an ordered list of string label indices,
      corresponding to the preferred display order in plots
      
    label_data: dict
      key: a string label index
      value: a dict with fields including "mnemonic" and "description",
        where description is a several-word description of the label
        and mnemonic is a few-character identifier
    """
    if filename is None:
        return []
    elif not os.path.isfile(filename):
        die("Could not find mnemonic file: %s" % filename)
        
    label_order = []
    label_data = {}
    with open(filename, "rU") as ifp:
        reader = DictReader(ifp)
        for row in reader:
            label_index = row["index"]
            label_order.append(label_index)
            label_data[label_index] = row
    
    return (label_order, label_data)

def key_by_field(fieldname):
    def entry_key(entry):
        return entry[fieldname]
    return entry_key

## Parses gff file for features
def load_gff_data(gff_filename):
    '''
    Expects data in GFF format (1-indexed locations):
    CHROM<tab>source<tab>feature<tab>START<tab>END<tab>score<tab>STRAND
    chrom, start, end, and strand are required
    strand can be '+'/'-' or '.', but not both in the same file

    Returns tssdata
    tssdata: a dict
      key: string chromosome name (e.g. "chr13")
      val: a list of dicts (ordered by the start index, ascending)
        key: "start", "end", or "strand"
        val: the associated data item, if it exists
             string for strand
             int (zero-indexed) for start and end (exclusive)
    '''
    data = defaultdict(list)
    stranded = None
    with open(gff_filename) as infile:
        for line in infile:
            tokens = line.strip().split("\t")
            chrom = tokens[0]

            name = tokens[2]
            start = int(tokens[3]) - 1  # Make zero-indexed
            # Make zero-indexed and exclusive (these cancel out)
            end = int(tokens[4])

            try:
                strand = tokens[6]
            except IndexError:
                strand = "."
                
            if strand == "+" or strand == "-":
                assert stranded or stranded is None
                stranded = True
            else:
                assert not stranded  # Don't have both +/- and .
                strand = "."  # N/A
                stranded = False
                
            feature = dict(name=name, start=start, end=end, strand=strand)
            data[chrom].append(feature)
            
    # Sort features by ascending start        
    for chrom_features in data.itervalues():
        chrom_features.sort(key=key_by_field("start"))
        
    return data

def load_segmentation(filename, checknames=True):
    """Returns a segmentation object derived from the given BED file

    If the labels in the BED file are all integers, the label_keys will
      be integer representations of those labels (unless checknames is False).
    If all of the labels are non-integers, the label_keys will be
      integers corresponding to the order observed.
    If there are some integer and some string labels, an error will be raised
      (unless checknames is False).
    """
    
    print >>sys.stderr, "Loading segmentation...",
    sys.stdout.flush()
    
    # first get the tracks that were used for this segmentation
    segtool, tracks = get_bed_metadata(filename)

    data = defaultdict(list)  # A dictionary-like object
    label_dict = {}
    # read in as lists of tuples
    with maybe_gzip_open(filename) as infile:
        for datum in read_native(infile):
            label = str(datum.name)
            try:  # Lookup label key
                label_key = label_dict[label]
            except KeyError:  # Map new label to key
                if checknames:
                    try:
                        label_key = int(label)
                    except ValueError:
                        checknames = False
                        if len(label_dict) > 0:  # Previous int labels found
                            die("Found both integer and string labels in BED file: %s" % filename)
                if not checknames:  # Sequential if's important
                    label_key = len(label_dict)
                    
                label_dict[label] = label_key
                
            segment = (datum.chromStart, datum.chromEnd, label_key)
            data[datum.chrom].append(segment)

    # Create reverse dict for return
    labels = dict((val, key) for key, val in label_dict.iteritems())
    print >>sys.stderr, ""
    for key, label in labels.iteritems():
        print >>sys.stderr, "%d -> \"%s\"" % (key, label)
            
    # Sort segments within each chromosome by start index
    for segments in data.itervalues():
        segments.sort()
        
    # convert lists of tuples to array
    chromosomes = dict((chrom, array(segments))
                       for chrom, segments in data.iteritems())

    print >>sys.stderr, "done."
    # wrap in a Segmentation object
    return Segmentation(chromosomes, labels, tracks, segtool)

## Returns a genome object from data in a genomedata directory
def load_genome(genomedatadir):
    if genomedatadir is not None and os.path.isdir(genomedatadir):
        genome = Genome(genomedatadir)
        if not genome or genome is None:
            die("Error: Unable to load genome data from directory: %s" % genomedatadir)
        else:
            return genome
    else:
        die("Error: Could not find genome data directory: %s" % genomedatadir)



#!/usr/bin/env python
from __future__ import division

__version__ = "$Revision: 84 $"

# Copyright 2008 Michael M. Hoffman <mmh1@washington.edu>

import sys

FIELDNAMES = ["chrom", "chromStart", "chromEnd", # required
              "name", "score", "strand", "thickStart", "thickEnd", "itemRgb",
              "blockCount", "blockSizes", "blockStarts"]

class Datum(object):
    def __init__(self, words):
        self.__dict__ = dict(zip(FIELDNAMES, words))
        self._words = tuple(words)

    def __repr__(self):
        return "%s%s" % (self.__class__.__name__, self._words)

class NativeDatum(Datum):
    def __init__(self, *args, **kwargs):
        Datum.__init__(self, *args, **kwargs)

        # zero-based, http://genome.ucsc.edu/FAQ/FAQformat#format1
        self.chromStart = int(self.chromStart)
        self.chromEnd = int(self.chromEnd)
        try:
            self.score = float(self.score)
        except AttributeError:
            pass
        except ValueError:
            pass

        self._words = ((self.chrom, self.chromStart, self.chromEnd)
                       + self._words[3:])
        
def read(iterator, datum_cls=Datum):
    for line in iterator:
        words = line.split()
        if words[0] == "track":  # Ignore any track lines
            continue

        assert len(words) >= 3

        yield datum_cls(words)


def read_native(*args, **kwargs):
    return read(datum_cls=NativeDatum, *args, **kwargs)

def main(args=sys.argv[1:]):
    pass

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python

"""segtools: tools for interpreting and validating segmentations

LONG_DESCRIPTION
"""

__version__ = "0.1.0"

# Copyright 2008-2009 Michael M. Hoffman <mmh1@washington.edu>

import sys

# required for from __future__ import division, with_statement;
# relative imports
assert sys.version_info >= (2, 5, 1)

from ez_setup import use_setuptools
use_setuptools()

from setuptools import find_packages, setup

doclines = __doc__.splitlines()
name, short_description = doclines[0].split(": ")
long_description = "\n".join(doclines[2:])

url = "http://noble.gs.washington.edu/~mmh1/software/%s/" % name.lower()
download_url = "%s%s-%s.tar.gz" % (url, name, __version__)

dependency_links = ["http://noble.gs.washington.edu/~mmh1/software/genomedata/",
                    "http://pypi.python.org/packages/source/p/path.py/path-2.2.zip"]

classifiers = ["Natural Language :: English",
               "Programming Language :: Python"]

entry_points = """
[console_scripts]
segtools-feature-aggregation = segtools.feature_aggregation:main
segtools-label-transition = segtools.label_transition:main
segtools-length-distribution = segtools.length_distribution:main
segtools-nucleotide-frequency = segtools.nucleotide_frequency:main
segtools-overlap = segtools.overlap:main
segtools-signal-distribution = segtools.signal_distribution:main
segtools-html-report = segtools.html:main
"""
#segtools = segtools.validate_all:main

install_requires = ["genomedata", "numpy", "path", "rpy2", "textinput", "tables>=2.1"]

setup(name=name,
      version=__version__,
      description=short_description,
      author="Michael Hoffman",
      author_email="mmh1@washington.edu",
      maintainer="Orion Buske",
      maintainer_email="orion.buske@gmail.com",
      url=url,
      download_url=download_url,
      classifiers=classifiers,
      long_description=long_description,
      dependency_links=dependency_links,
      install_requires=install_requires,
      zip_safe=False,
      packages=find_packages("."),
      include_package_data=True,
      entry_points=entry_points
      )
